@@ -33,7 +33,7 @@
 
 #include <vector>
 
-#include <gtkmm/treemodelfilter.h>
+#include <ytkmm/treemodelfilter.h>
 
 #include "pbd/convert.h"
 
@@ -61,6 +61,8 @@
 #include "gui_object.h"
 #include "gui_thread.h"
 #include "keyeditor.h"
+#include "strip_export_dialog.h"
+#include "strip_import_dialog.h"
 #include "library_download_dialog.h"
 #include "location_ui.h"
 #include "lua_script_manager.h"
@@ -70,8 +72,8 @@
 #include "meter_patterns.h"
 #include "monitor_section.h"
 #include "midi_tracer.h"
-#include "mini_timeline.h"
 #include "mixer_ui.h"
+#include "pianoroll_window.h"
 #include "plugin_dspload_window.h"
 #include "plugin_manager_ui.h"
 #include "public_editor.h"
@@ -79,12 +81,12 @@
 #include "rc_option_editor.h"
 #include "recorder_ui.h"
 #include "route_params_ui.h"
+#include "rta_window.h"
 #include "shuttle_control.h"
 #include "session_option_editor.h"
 #include "speaker_dialog.h"
 #include "splash.h"
 #include "sfdb_ui.h"
-#include "time_info_box.h"
 #include "timers.h"
 #include "transport_masters_dialog.h"
 #include "trigger_page.h"
@@ -123,13 +125,10 @@ ARDOUR_UI::set_session (Session *s)
 		ActionManager::set_sensitive (ActionManager::range_sensitive_actions, false);
 	}
 
-	transport_ctrl.set_session (s);
-
 	update_path_label ();
 	update_sample_rate ();
 
 	if (!_session) {
-		WM::Manager::instance().set_session (s);
 		/* Session option editor cannot exist across change-of-session */
 		session_option_editor.drop_window ();
 		/* Ditto for AddVideoDialog */
@@ -156,10 +155,6 @@ ARDOUR_UI::set_session (Session *s)
 
 	AutomationWatch::instance().set_session (s);
 
-	shuttle_box.set_session (s);
-	mini_timeline.set_session (s);
-	time_info_box->set_session (s);
-
 	primary_clock->set_session (s);
 	secondary_clock->set_session (s);
 	big_clock->set_session (s);
@@ -193,31 +188,25 @@ ARDOUR_UI::set_session (Session *s)
 	ActionManager::set_sensitive (ActionManager::point_selection_sensitive_actions, false);
 	ActionManager::set_sensitive (ActionManager::playlist_selection_sensitive_actions, false);
 
-	solo_alert_button.set_active (_session->soloing());
-
 	setup_session_options ();
 
 	blink_connection = Timers::blink_connect (sigc::mem_fun(*this, &ARDOUR_UI::blink_handler));
 
-	_session->SaveSessionRequested.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::save_session_at_its_request, this, _1), gui_context());
-	_session->StateSaved.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::update_title, this), gui_context());
-	_session->StateSaved.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::update_path_label, this), gui_context());
-	_session->RecordStateChanged.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::record_state_changed, this), gui_context());
-	_session->TransportStateChange.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::map_transport_state, this), gui_context());
-	_session->DirtyChanged.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::session_dirty_changed, this), gui_context());
+	_session->SaveSessionRequested.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::save_session_at_its_request, this, _1), gui_context());
+	_session->StateSaved.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::update_title, this), gui_context());
+	_session->StateSaved.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::update_path_label, this), gui_context());
+	_session->RecordStateChanged.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::record_state_changed, this), gui_context());
+	_session->TransportStateChange.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::map_transport_state, this), gui_context());
+	_session->DirtyChanged.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::session_dirty_changed, this), gui_context());
+	_session->LatencyUpdated.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::session_latency_updated, this, _1), gui_context());
 
-	_session->PunchLoopConstraintChange.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::set_punch_sensitivity, this), gui_context());
-	_session->auto_punch_location_changed.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::set_punch_sensitivity, this), gui_context ());
+	_session->PunchLoopConstraintChange.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::set_punch_sensitivity, this), gui_context());
+	_session->auto_punch_location_changed.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::set_punch_sensitivity, this), gui_context ());
 
-	_session->Xrun.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::xrun_handler, this, _1), gui_context());
-	_session->SoloActive.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::soloing_changed, this, _1), gui_context());
-	_session->AuditionActive.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::auditioning_changed, this, _1), gui_context());
-	_session->locations()->added.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::handle_locations_change, this, _1), gui_context());
-	_session->locations()->removed.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::handle_locations_change, this, _1), gui_context());
-	_session->config.ParameterChanged.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::session_parameter_changed, this, _1), gui_context ());
-
-	_session->LatencyUpdated.connect (_session_connections, MISSING_INVALIDATOR, boost::bind (&ARDOUR_UI::session_latency_updated, this, _1), gui_context());
-	session_latency_updated (true);
+	_session->Xrun.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::xrun_handler, this, _1), gui_context());
+	_session->locations()->added.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::handle_locations_change, this, _1), gui_context());
+	_session->locations()->removed.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::handle_locations_change, this, _1), gui_context());
+	_session->config.ParameterChanged.connect (_session_connections, MISSING_INVALIDATOR, std::bind (&ARDOUR_UI::session_parameter_changed, this, _1), gui_context ());
 
 	/* Clocks are on by default after we are connected to a session, so show that here.
 	*/
@@ -238,60 +227,10 @@ ARDOUR_UI::set_session (Session *s)
 
 	second_connection = Timers::second_connect (sigc::mem_fun(*this, &ARDOUR_UI::every_second));
 	point_one_second_connection = Timers::rapid_connect (sigc::mem_fun(*this, &ARDOUR_UI::every_point_one_seconds));
-	point_zero_something_second_connection = Timers::super_rapid_connect (sigc::mem_fun(*this, &ARDOUR_UI::every_point_zero_something_seconds));
 	set_fps_timeout_connection();
 
 	update_format ();
 
-	if (editor_meter_table.get_parent()) {
-		transport_hbox.remove (editor_meter_table);
-	}
-
-	if (editor_meter) {
-		editor_meter_table.remove(*editor_meter);
-		delete editor_meter;
-		editor_meter = 0;
-	}
-
-	if (editor_meter_table.get_parent()) {
-		transport_hbox.remove (editor_meter_table);
-	}
-	if (editor_meter_peak_display.get_parent ()) {
-		editor_meter_table.remove (editor_meter_peak_display);
-	}
-
-	if (_session &&
-	    _session->master_out() &&
-	    _session->master_out()->n_outputs().n(DataType::AUDIO) > 0) {
-
-		editor_meter = new LevelMeterHBox(_session);
-		editor_meter->set_meter (_session->master_out()->shared_peak_meter().get());
-		editor_meter->clear_meters();
-		editor_meter->setup_meters (30, 10, 6);
-		editor_meter->show();
-
-		editor_meter_table.set_spacings(3);
-		editor_meter_table.attach(*editor_meter,             0,1, 0,1, FILL, EXPAND|FILL, 0, 1);
-		editor_meter_table.attach(editor_meter_peak_display, 0,1, 1,2, FILL, SHRINK, 0, 0);
-
-		editor_meter->show();
-		editor_meter_peak_display.show();
-
-		ArdourMeter::ResetAllPeakDisplays.connect (sigc::mem_fun(*this, &ARDOUR_UI::reset_peak_display));
-		ArdourMeter::ResetRoutePeakDisplays.connect (sigc::mem_fun(*this, &ARDOUR_UI::reset_route_peak_display));
-		ArdourMeter::ResetGroupPeakDisplays.connect (sigc::mem_fun(*this, &ARDOUR_UI::reset_group_peak_display));
-
-		editor_meter_peak_display.set_name ("meterbridge peakindicator");
-		editor_meter_peak_display.set_can_focus (false);
-		editor_meter_peak_display.set_size_request (-1, std::max (5.f, std::min (12.f, rintf (8.f * UIConfiguration::instance().get_ui_scale()))) );
-		editor_meter_peak_display.set_corner_radius (1.0);
-
-		_clear_editor_meter = true;
-		editor_meter_peak_display.signal_button_release_event().connect (sigc::mem_fun(*this, &ARDOUR_UI::editor_meter_peak_button_release), false);
-
-		repack_transport_hbox ();
-	}
-
 	update_title ();
 }
 
@@ -367,20 +306,10 @@ ARDOUR_UI::unload_session (bool hide_stuff, bool force_unload)
 
 	second_connection.disconnect ();
 	point_one_second_connection.disconnect ();
-	point_zero_something_second_connection.disconnect();
 	fps_connection.disconnect();
 
-	if (editor_meter) {
-		editor_meter_table.remove(*editor_meter);
-		delete editor_meter;
-		editor_meter = 0;
-		editor_meter_peak_display.hide();
-	}
-
 	ActionManager::set_sensitive (ActionManager::session_sensitive_actions, false);
 
-	WM::Manager::instance().set_session ((ARDOUR::Session*) 0);
-
 	if (ARDOUR_UI::instance()->video_timeline) {
 		ARDOUR_UI::instance()->video_timeline->close_session();
 	}
@@ -698,8 +627,26 @@ ARDOUR_UI::tabs_page_removed (Widget*, guint)
 void
 ARDOUR_UI::tabs_switch (GtkNotebookPage*, guint page)
 {
+	if (tabbables_table.get_parent ()) {
+		if (editor) {
+			editor->tab_btn_box ().remove ();
+		}
+		if (mixer) {
+			mixer->tab_btn_box ().remove ();
+		}
+		if (recorder) {
+			recorder->tab_btn_box ().remove ();
+		}
+		if (trigger_page) {
+			trigger_page->tab_btn_box ().remove ();
+		}
+	}
+
+	//pack the tabbables selector in this tab, and set button sensitivity appropriately
 	if (editor && (page == (guint) _tabs.page_num (editor->contents()))) {
 
+		editor->tab_btn_box ().add (tabbables_table);
+
 		editor_visibility_button.set_active_state (Gtkmm2ext::ImplicitActive);
 
 		if (mixer && (mixer->tabbed() || mixer->tabbed_by_default())) {
@@ -720,6 +667,8 @@ ARDOUR_UI::tabs_switch (GtkNotebookPage*, guint page)
 
 	} else if (mixer && (page == (guint) _tabs.page_num (mixer->contents()))) {
 
+		mixer->tab_btn_box ().add (tabbables_table);
+
 		if (editor && (editor->tabbed() || editor->tabbed_by_default())) {
 			editor_visibility_button.set_active_state (Gtkmm2ext::Off);
 		}
@@ -760,6 +709,8 @@ ARDOUR_UI::tabs_switch (GtkNotebookPage*, guint page)
 
 	} else if (page == (guint) _tabs.page_num (recorder->contents())) {
 
+		recorder->tab_btn_box ().add (tabbables_table);
+
 		if (editor && (editor->tabbed() || editor->tabbed_by_default())) {
 			editor_visibility_button.set_active_state (Gtkmm2ext::Off);
 		}
@@ -780,6 +731,8 @@ ARDOUR_UI::tabs_switch (GtkNotebookPage*, guint page)
 
 	} else if (page == (guint) _tabs.page_num (trigger_page->contents())) {
 
+		trigger_page->tab_btn_box ().add (tabbables_table);
+
 		if (editor && (editor->tabbed() || editor->tabbed_by_default())) {
 			editor_visibility_button.set_active_state (Gtkmm2ext::Off);
 		}
@@ -963,7 +916,7 @@ ARDOUR_UI::new_midi_tracer_window ()
 		t->show_all ();
 		_midi_tracer_windows.push_back (t);
 	} else {
-		/* re-use the hidden one */
+		/* reuse the hidden one */
 		(*i)->show_all ();
 	}
 }
@@ -973,8 +926,10 @@ ARDOUR_UI::create_key_editor ()
 {
 	KeyEditor* kedit = new KeyEditor;
 
-	for (std::list<Bindings*>::iterator b = Bindings::bindings.begin(); b != Bindings::bindings.end(); ++b) {
-		kedit->add_tab ((*b)->name(), **b);
+	for (auto & b : Bindings::bindings) {
+		if (!b->parent()) {
+			kedit->add_tab (b->name(), *b);
+		}
 	}
 
 	return kedit;
@@ -1038,6 +993,38 @@ ARDOUR_UI::create_luawindow ()
 	return luawindow;
 }
 
+RTAWindow*
+ARDOUR_UI::create_rtawindow ()
+{
+	RTAWindow* rtawindow = new RTAWindow ();
+	return rtawindow;
+}
+
+void ARDOUR_UI::export_strips ()
+{
+	if (_session) {
+		StripExportDialog esd (*editor, _session);
+		esd.run();
+	}
+}
+
+void ARDOUR_UI::import_strips ()
+{
+	if (_session) {
+		StripImportDialog isd (_session);
+		switch (isd.run()) {
+			default:
+				break;
+			case RESPONSE_ACCEPT:
+				DisplaySuspender ds;
+				Mixer_UI::RedisplaySuspender rs;
+				PresentationInfo::ChangeSuspender cs;
+				isd.do_import ();
+				break;
+		}
+	}
+}
+
 void
 ARDOUR_UI::handle_locations_change (Location *)
 {
@@ -1089,15 +1076,6 @@ ARDOUR_UI::tabbed_window_state_event_handler (GdkEventWindowState* ev, void* obj
 	return false;
 }
 
-bool
-ARDOUR_UI::editor_meter_peak_button_release (GdkEventButton* ev)
-{
-	if (ev->button == 1) {
-		ArdourMeter::ResetAllPeakDisplays ();
-	}
-	return false;
-}
-
 void
 ARDOUR_UI::toggle_mixer_space()
 {
@@ -1108,6 +1086,12 @@ ARDOUR_UI::toggle_mixer_space()
 		mixer->restore_mixer_space ();
 	}
 }
+void
+ARDOUR_UI::show_lua_window ()
+{
+	Glib::RefPtr<ToggleAction> tact = ActionManager::get_toggle_action ("Window", "toggle-luawindow");
+	tact->set_active();
+}
 
 void
 ARDOUR_UI::show_plugin_manager ()
@@ -1116,6 +1100,13 @@ ARDOUR_UI::show_plugin_manager ()
 	tact->set_active();
 }
 
+void
+ARDOUR_UI::show_realtime_analyzer ()
+{
+	Glib::RefPtr<ToggleAction> tact = ActionManager::get_toggle_action ("Window", "toggle-rtawindow");
+	tact->set_active();
+}
+
 bool
 ARDOUR_UI::timecode_button_press (GdkEventButton* ev)
 {
