@@ -27,6 +27,8 @@
 #include "gtk2ardour-config.h"
 #endif
 
+#include <boost/algorithm/string.hpp>
+
 #include "gtkmm2ext/utils.h"
 
 #include "ardour/profile.h"
@@ -45,6 +47,7 @@
 
 #include "ardour_ui.h"
 #include "automation_time_axis.h"
+#include "control_point.h"
 #include "editor.h"
 #include "editing.h"
 #include "rgb_macros.h"
@@ -94,6 +97,8 @@ Editor::initialize_canvas ()
 	*/
 	no_scroll_group = new ArdourCanvas::Container (_track_canvas->root());
 
+	_verbose_cursor.reset (new VerboseCursor (*this));
+
 	ArdourCanvas::ScrollGroup* hsg;
 	ArdourCanvas::ScrollGroup* hg;
 	ArdourCanvas::ScrollGroup* cg;
@@ -112,7 +117,6 @@ Editor::initialize_canvas ()
 	CANVAS_DEBUG_NAME (cursor_scroll_group, "canvas cursor scroll");
 	_track_canvas->add_scroller (*cg);
 
-	_verbose_cursor = new VerboseCursor (this);
 	_region_peak_cursor = new RegionPeakCursor (get_noscroll_group ());
 
 	/*a group to hold global rects like punch/loop indicators */
@@ -170,10 +174,6 @@ Editor::initialize_canvas ()
 	meter_group = new ArdourCanvas::Container (_time_markers_group, ArdourCanvas::Duple (0.0, (timebar_height * 5.0) + 1.0));
 	CANVAS_DEBUG_NAME (meter_group, "meter group");
 
-	float timebar_thickness = timebar_height; //was 4
-	float timebar_top = (timebar_height - timebar_thickness)/2;
-	float timebar_btm = timebar_height - timebar_top;
-
 	meter_bar = new ArdourCanvas::Rectangle (meter_group, ArdourCanvas::Rect (0.0, 0., ArdourCanvas::COORD_MAX, timebar_height));
 	CANVAS_DEBUG_NAME (meter_bar, "meter Bar");
 	meter_bar->set_outline(false);
@@ -184,13 +184,16 @@ Editor::initialize_canvas ()
 	tempo_bar->set_outline(false);
 	tempo_bar->set_outline_what(ArdourCanvas::Rectangle::BOTTOM);
 
-	range_marker_bar = new ArdourCanvas::Rectangle (range_marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
+	range_marker_bar = new ArdourCanvas::Rectangle (range_marker_group, ArdourCanvas::Rect (0.0, 0, ArdourCanvas::COORD_MAX, timebar_height));
+	range_marker_bar->set_outline_what(ArdourCanvas::Rectangle::BOTTOM);
 	CANVAS_DEBUG_NAME (range_marker_bar, "Range Marker Bar");
 
-	marker_bar = new ArdourCanvas::Rectangle (marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
+	marker_bar = new ArdourCanvas::Rectangle (marker_group, ArdourCanvas::Rect (0.0, 0, ArdourCanvas::COORD_MAX, timebar_height));
+	marker_bar->set_outline_what(ArdourCanvas::Rectangle::BOTTOM);
 	CANVAS_DEBUG_NAME (marker_bar, "Marker Bar");
 
-	section_marker_bar = new ArdourCanvas::Rectangle (section_marker_group, ArdourCanvas::Rect (0.0, timebar_top, ArdourCanvas::COORD_MAX, timebar_btm));
+	section_marker_bar = new ArdourCanvas::Rectangle (section_marker_group, ArdourCanvas::Rect (0.0, 0, ArdourCanvas::COORD_MAX, timebar_height));
+	section_marker_bar->set_outline_what(ArdourCanvas::Rectangle::BOTTOM);
 	CANVAS_DEBUG_NAME (section_marker_bar, "Arranger Marker Bar");
 
 	ruler_separator = new ArdourCanvas::Line(_time_markers_group);
@@ -228,7 +231,7 @@ Editor::initialize_canvas ()
 	videotl_group->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_videotl_bar_event), videotl_group));
 	range_marker_bar->Event.connect (sigc::bind (sigc::mem_fun (*this, &Editor::canvas_ruler_bar_event), range_marker_bar, RangeMarkerBarItem, "range marker bar"));
 
-	_playhead_cursor = new EditorCursor (*this, &Editor::canvas_playhead_cursor_event, X_("playhead"));
+	_playhead_cursor = new EditorCursor (*this, &EditingContext::canvas_playhead_cursor_event, X_("playhead"));
 	_playhead_cursor->set_sensitive (UIConfiguration::instance().get_sensitize_playhead());
 
 	_snapped_cursor = new EditorCursor (*this, X_("snapped"));
@@ -275,13 +278,13 @@ Editor::initialize_canvas ()
 	_track_canvas->signal_enter_notify_event().connect (sigc::mem_fun(*this, &Editor::entered_track_canvas), false);
 	_track_canvas->set_can_focus ();
 
-	_track_canvas->PreRender.connect (sigc::mem_fun(*this, &Editor::pre_render));
+	_track_canvas->PreRender.connect (sigc::mem_fun(*this, &EditingContext::pre_render));
 
 	/* set up drag-n-drop */
 
 	vector<TargetEntry> target_table;
 
-	target_table.push_back (TargetEntry ("x-ardour/region.pbdid", TARGET_SAME_APP));
+	target_table.push_back (TargetEntry ("x-ardour/region.pbdids", TARGET_SAME_APP));
 	target_table.push_back (TargetEntry ("text/uri-list"));
 	target_table.push_back (TargetEntry ("text/plain"));
 	target_table.push_back (TargetEntry ("application/x-rootwin-drop"));
@@ -294,8 +297,8 @@ Editor::initialize_canvas ()
 	initialize_rulers ();
 
 	UIConfiguration::instance().ColorsChanged.connect (sigc::mem_fun (*this, &Editor::color_handler));
+	UIConfiguration::instance().DPIReset.connect (sigc::mem_fun (*this, &Editor::dpi_reset));
 	color_handler();
-
 }
 
 void
@@ -312,6 +315,7 @@ Editor::track_canvas_viewport_size_allocated ()
 
 	_visible_canvas_width  = _canvas_viewport_allocation.get_width ();
 	_visible_canvas_height = _canvas_viewport_allocation.get_height ();
+	_track_canvas_width = _visible_canvas_width;
 
 	_canvas_drop_zone->set_y1 (_canvas_drop_zone->y0() + (_visible_canvas_height - 20.0));
 
@@ -377,16 +381,13 @@ Editor::reset_controls_layout_height (int32_t h)
 	controls_layout.property_height() = h;
 
 	_group_tabs->set_extent (h);
+	controls_layout.queue_draw ();
 }
 
 bool
 Editor::track_canvas_map_handler (GdkEventAny* /*ev*/)
 {
-	if (!_cursor_stack.empty()) {
-		set_canvas_cursor (get_canvas_cursor());
-	} else {
-		PBD::error << "cursor stack is empty" << endmsg;
-	}
+	set_canvas_cursor (get_canvas_cursor());
 	return false;
 }
 
@@ -401,7 +402,21 @@ Editor::track_canvas_drag_data_received (const RefPtr<Gdk::DragContext>& context
 		return;
 	}
 	if (data.get_target() == "x-ardour/region.pbdid") {
-		drop_regions (context, x, y, data, info, time);
+		PBD::ID rid (data.get_data_as_string ());
+		drop_region (context, x, y, rid, info, time);
+	} else if (data.get_target() == "x-ardour/region.pbdids") {
+		std::vector<std::string> ids;
+		std::vector<PBD::ID> rids;
+		std::string const& data_str (data.get_data_as_string ());
+		boost::split (ids, data_str, boost::is_any_of(","));
+		for (auto const& id: ids) {
+			rids.push_back (id);
+		}
+		if (rids.size () == 1) {
+			drop_region (context, x, y, rids.front (), info, time);
+		} else {
+			drop_regions (context, x, y, rids, info, time);
+		}
 	} else {
 		drop_paths (context, x, y, data, info, time);
 	}
@@ -599,18 +614,6 @@ Editor::maybe_autoscroll (bool allow_horiz, bool allow_vert, bool from_headers)
 	}
 }
 
-bool
-Editor::drag_active () const
-{
-	return _drags->active();
-}
-
-bool
-Editor::preview_video_drag_active () const
-{
-	return _drags->preview_video ();
-}
-
 bool
 Editor::autoscroll_active () const
 {
@@ -651,6 +654,11 @@ Editor::session_gui_extents (bool use_extra) const
 		}
 	}
 
+	if (_session->actively_recording ()) {
+		session_extent_start = std::min (session_extent_start, timepos_t (_session->last_transport_start ()));
+		session_extent_end   = std::max (session_extent_end, timepos_t (_session->transport_sample ()));
+	}
+
 	/* ToDo: also incorporate automation regions (in case the session has no audio/midi but is just used for automating plugins or the like) */
 
 	/* add additional time to the ui extents (user-defined in config) */
@@ -893,25 +901,14 @@ Editor::stop_canvas_autoscroll ()
 	autoscroll_cnt = 0;
 }
 
-Editor::EnterContext*
-Editor::get_enter_context(ItemType type)
-{
-	for (ssize_t i = _enter_stack.size() - 1; i >= 0; --i) {
-		if (_enter_stack[i].item_type == type) {
-			return &_enter_stack[i];
-		}
-	}
-	return NULL;
-}
-
 bool
 Editor::left_track_canvas (GdkEventCrossing* ev)
 {
 	const bool was_within = within_track_canvas;
 	DropDownKeys ();
 	within_track_canvas = false;
-	set_entered_track (0);
-	set_entered_regionview (0);
+	set_entered_track (nullptr);
+	set_entered_regionview (nullptr);
 	reset_canvas_action_sensitivity (false);
 
 	if (was_within) {
@@ -936,13 +933,7 @@ Editor::entered_track_canvas (GdkEventCrossing* ev)
 
 	if (!was_within) {
 
-		if (internal_editing()) {
-			/* ensure that key events go here because there are
-			   internal editing bindings associated only with the
-			   canvas. if the focus is elsewhere, we cannot find them.
-			*/
-			_track_canvas->grab_focus ();
-		}
+		_track_canvas->grab_focus ();
 
 		if (ev->detail == GDK_NOTIFY_NONLINEAR ||
 		    ev->detail == GDK_NOTIFY_NONLINEAR_VIRTUAL) {
@@ -1013,14 +1004,7 @@ Editor::tie_vertical_scrolling ()
 		_summary->set_overlays_dirty ();
 	}
 	_group_tabs->set_offset (vertical_adjustment.get_value ());
-}
-
-void
-Editor::set_horizontal_position (double p)
-{
-	horizontal_adjustment.set_value (p);
-
-	_leftmost_sample = (samplepos_t) floor (p * samples_per_pixel);
+	controls_layout.queue_draw ();
 }
 
 void
@@ -1088,10 +1072,16 @@ Editor::color_handler()
 */
 }
 
-double
-Editor::horizontal_position () const
+ArdourCanvas::GtkCanvasViewport*
+Editor::get_canvas_viewport() const
 {
-	return sample_to_pixel (_leftmost_sample);
+	return _track_canvas_viewport;
+}
+
+ArdourCanvas::GtkCanvas*
+Editor::get_canvas() const
+{
+	return _track_canvas_viewport->canvas();
 }
 
 bool
@@ -1125,66 +1115,6 @@ Editor::clamp_verbose_cursor_y (double y)
 	return y;
 }
 
-ArdourCanvas::GtkCanvasViewport*
-Editor::get_track_canvas() const
-{
-	return _track_canvas_viewport;
-}
-
-Gdk::Cursor*
-Editor::get_canvas_cursor () const
-{
-	/* The top of the cursor stack is always the currently visible cursor. */
-	return _cursor_stack.back();
-}
-
-void
-Editor::set_canvas_cursor (Gdk::Cursor* cursor)
-{
-	Glib::RefPtr<Gdk::Window> win = _track_canvas->get_window();
-
-	if (win && !_cursors->is_invalid (cursor)) {
-		/* glibmm 2.4 doesn't allow null cursor pointer because it uses
-		   a Gdk::Cursor& as the argument to Gdk::Window::set_cursor().
-		   But a null pointer just means "use parent window cursor",
-		   and so should be allowed. Gtkmm 3.x has fixed this API.
-
-		   For now, drop down and use C API
-		*/
-		gdk_window_set_cursor (win->gobj(), cursor ? cursor->gobj() : 0);
-	}
-}
-
-size_t
-Editor::push_canvas_cursor (Gdk::Cursor* cursor)
-{
-	if (!_cursors->is_invalid (cursor)) {
-		_cursor_stack.push_back (cursor);
-		set_canvas_cursor (cursor);
-	}
-	return _cursor_stack.size() - 1;
-}
-
-void
-Editor::pop_canvas_cursor ()
-{
-	while (true) {
-		if (_cursor_stack.size() <= 1) {
-			PBD::error << "attempt to pop default cursor" << endmsg;
-			return;
-		}
-
-		_cursor_stack.pop_back();
-		if (_cursor_stack.back()) {
-			/* Popped to an existing cursor, we're done.  Otherwise, the
-			   context that created this cursor has been destroyed, so we need
-			   to skip to the next down the stack. */
-			set_canvas_cursor (_cursor_stack.back());
-			return;
-		}
-	}
-}
-
 Gdk::Cursor*
 Editor::which_trim_cursor (bool left) const
 {
@@ -1214,7 +1144,7 @@ Editor::which_mode_cursor () const
 {
 	Gdk::Cursor* mode_cursor = MouseCursors::invalid_cursor ();
 
-	switch (mouse_mode) {
+	switch (current_mouse_mode()) {
 	case MouseRange:
 		mode_cursor = _cursors->selector;
 		break;
@@ -1290,10 +1220,21 @@ Editor::which_track_cursor () const
 	return cursor;
 }
 
+double
+Editor::trackviews_height() const
+{
+	if (!_trackview_group) {
+		return 0;
+	}
+
+	return _visible_canvas_height - _trackview_group->canvas_origin().y;
+}
+
 Gdk::Cursor*
 Editor::which_canvas_cursor(ItemType type) const
 {
 	Gdk::Cursor* cursor = which_mode_cursor ();
+	auto mouse_mode = current_mouse_mode();
 
 	if (mouse_mode == MouseRange) {
 		switch (type) {
@@ -1314,11 +1255,7 @@ Editor::which_canvas_cursor(ItemType type) const
 		/* find correct cursor to use in object/smart mode */
 		switch (type) {
 		case RegionItem:
-		/* We don't choose a cursor for these items on top of a region view,
-		   because this would push a new context on the enter stack which
-		   means switching the region context for things like smart mode
-		   won't actually change the cursor. */
-		// case WaveItem:
+		case WaveItem:
 		case StreamItem:
 		case AutomationTrackItem:
 			cursor = which_track_cursor ();
@@ -1335,7 +1272,7 @@ Editor::which_canvas_cursor(ItemType type) const
 		case GainLineItem:
 			cursor = _cursors->cross_hair;
 			break;
-		case AutomationLineItem:
+		case EditorAutomationLineItem:
 			cursor = _cursors->cross_hair;
 			break;
 		case StartSelectionTrimItem:
@@ -1366,10 +1303,11 @@ Editor::which_canvas_cursor(ItemType type) const
 			cursor = _cursors->cross_hair;
 			break;
 		case LeftFrameHandle:
-			if (effective_mouse_mode() == MouseObject) // (smart mode): if the user is in the btm half, show the trim cursor
+			if (effective_mouse_mode() == MouseObject) {// (smart mode): if the user is in the btm half, show the trim cursor
 				cursor = which_trim_cursor (true);
-			else
+			} else {
 				cursor = _cursors->selector; // (smart mode): in the top half, just show the selection (range) cursor
+			}
 			break;
 		case RightFrameHandle:
 			if (effective_mouse_mode() == MouseObject) // see above
@@ -1460,36 +1398,239 @@ Editor::which_canvas_cursor(ItemType type) const
 	return cursor;
 }
 
-void
-Editor::choose_canvas_cursor_on_entry (ItemType type)
+bool
+Editor::enter_handler (ArdourCanvas::Item* item, GdkEvent* event, ItemType item_type)
 {
-	if (_drags->active()) {
-		return;
+	ControlPoint* cp;
+	ArdourMarker * marker;
+	MeterMarker* m_marker = 0;
+	TempoMarker* t_marker = 0;
+	double fraction;
+	bool ret = true;
+	auto mouse_mode = current_mouse_mode();
+
+	/* by the time we reach here, entered_regionview and entered trackview
+	 * will have already been set as appropriate. Things are done this
+	 * way because this method isn't passed a pointer to a variable type of
+	 * thing that is entered (which may or may not be canvas item).
+	 * (e.g. the actual entered regionview)
+	 */
+
+	choose_canvas_cursor_on_entry (item_type);
+
+	switch (item_type) {
+	case GridZoneItem:
+		break;
+
+	case ControlPointItem:
+		if (mouse_mode == MouseDraw || mouse_mode == MouseObject || mouse_mode == MouseContent) {
+			cp = static_cast<ControlPoint*>(item->get_data ("control_point"));
+			cp->show ();
+
+			fraction = 1.0 - (cp->get_y() / cp->line().height());
+
+			_verbose_cursor->set (cp->line().get_verbose_cursor_string (fraction));
+			_verbose_cursor->show ();
+		}
+		break;
+
+	case GainLineItem:
+		if (mouse_mode == MouseDraw) {
+			ArdourCanvas::Line *line = dynamic_cast<ArdourCanvas::Line *> (item);
+			if (line) {
+				line->set_outline_color (UIConfiguration::instance().color ("entered gain line"));
+			}
+		}
+		break;
+
+	case EditorAutomationLineItem:
+		if (mouse_mode == MouseDraw || mouse_mode == MouseObject) {
+			ArdourCanvas::Line *line = dynamic_cast<ArdourCanvas::Line *> (item);
+			if (line) {
+				line->set_outline_color (UIConfiguration::instance().color ("entered automation line"));
+			}
+		}
+		break;
+
+	case AutomationTrackItem:
+		AutomationTimeAxisView* atv;
+		if ((atv = static_cast<AutomationTimeAxisView*>(item->get_data ("trackview"))) != 0) {
+			clear_entered_track = false;
+			set_entered_track (atv);
+		}
+		break;
+
+	case MarkerItem:
+		if ((marker = static_cast<ArdourMarker *> (item->get_data ("marker"))) == 0) {
+			break;
+		}
+		entered_marker = marker;
+		marker->set_entered (true);
+		break;
+
+	case MeterMarkerItem:
+		if ((m_marker = static_cast<MeterMarker *> (item->get_data ("marker"))) == 0) {
+			break;
+		}
+		entered_marker = m_marker;
+		/* "music" currently serves as a stand-in for "entered". */
+		m_marker->set_color ("meter marker music");
+		break;
+
+	case TempoMarkerItem:
+		if ((t_marker = static_cast<TempoMarker *> (item->get_data ("marker"))) == 0) {
+			break;
+		}
+		entered_marker = t_marker;
+		/* "music" currently serves as a stand-in for "entered". */
+		t_marker->set_color ("tempo marker music");
+		break;
+
+	case FadeInHandleItem:
+	case FadeInTrimHandleItem:
+		if (mouse_mode == MouseObject) {
+			ArdourCanvas::Rectangle *rect = dynamic_cast<ArdourCanvas::Rectangle *> (item);
+			if (rect) {
+				RegionView* rv = static_cast<RegionView*>(item->get_data ("regionview"));
+				rect->set_fill_color (rv->get_fill_color());
+			}
+		}
+		break;
+
+	case FadeOutHandleItem:
+	case FadeOutTrimHandleItem:
+		if (mouse_mode == MouseObject) {
+			ArdourCanvas::Rectangle *rect = dynamic_cast<ArdourCanvas::Rectangle *> (item);
+			if (rect) {
+				RegionView* rv = static_cast<RegionView*>(item->get_data ("regionview"));
+				rect->set_fill_color (rv->get_fill_color ());
+			}
+		}
+		break;
+
+	case FeatureLineItem:
+	{
+		ArdourCanvas::Line *line = dynamic_cast<ArdourCanvas::Line *> (item);
+		line->set_outline_color (0xFF0000FF);
+	}
+	break;
+
+	case SelectionItem:
+		break;
+
+	case WaveItem:
+	{
+		if (entered_regionview) {
+			entered_regionview->entered();
+		}
+	}
+	break;
+
+	default:
+		break;
 	}
 
-	Gdk::Cursor* cursor = which_canvas_cursor(type);
+	/* third pass to handle entered track status in a comprehensible way.
+	 */
 
-	if (!_cursors->is_invalid (cursor)) {
-		// Push a new enter context
-		const EnterContext ctx = { type, CursorContext::create(*this, cursor) };
-		_enter_stack.push_back(ctx);
+	switch (item_type) {
+	case GainLineItem:
+	case EditorAutomationLineItem:
+	case ControlPointItem:
+		/* these do not affect the current entered track state */
+		clear_entered_track = false;
+		break;
+
+	case AutomationTrackItem:
+		/* handled above already */
+		break;
+
+	default:
+
+		break;
 	}
+
+	return ret;
 }
 
-void
-Editor::update_all_enter_cursors ()
+bool
+Editor::leave_handler (ArdourCanvas::Item* item, GdkEvent*, ItemType item_type)
 {
-	for (auto & ec : _enter_stack) {
-		ec.cursor_ctx->change(which_canvas_cursor (ec.item_type));
-	}
-}
+	EditorAutomationLine* al;
+	ArdourMarker *marker;
+	TempoMarker *t_marker;
+	MeterMarker *m_marker;
+	bool ret = true;
 
-double
-Editor::trackviews_height() const
-{
-	if (!_trackview_group) {
-		return 0;
+	switch (item_type) {
+	case GridZoneItem:
+		break;
+
+	case ControlPointItem:
+		_verbose_cursor->hide ();
+		break;
+
+	case GainLineItem:
+	case EditorAutomationLineItem:
+		al = reinterpret_cast<EditorAutomationLine*> (item->get_data ("line"));
+		{
+			ArdourCanvas::Line *line = dynamic_cast<ArdourCanvas::Line *> (item);
+			if (line) {
+				line->set_outline_color (al->get_line_color());
+			}
+		}
+		break;
+
+	case MarkerItem:
+		if ((marker = static_cast<ArdourMarker *> (item->get_data ("marker"))) == 0) {
+			break;
+		}
+		entered_marker = 0;
+		marker->set_entered (false);
+		break;
+
+	case MeterMarkerItem:
+		if ((m_marker = static_cast<MeterMarker *> (item->get_data ("marker"))) == 0) {
+			break;
+		}
+		m_marker->set_color ("meter marker");
+		entered_marker = 0;
+		break;
+
+	case TempoMarkerItem:
+		if ((t_marker = static_cast<TempoMarker *> (item->get_data ("marker"))) == 0) {
+			break;
+		}
+		t_marker->set_color ("tempo marker");
+		entered_marker = 0;
+		break;
+
+	case FadeInTrimHandleItem:
+	case FadeOutTrimHandleItem:
+	case FadeInHandleItem:
+	case FadeOutHandleItem:
+	{
+		ArdourCanvas::Rectangle *rect = dynamic_cast<ArdourCanvas::Rectangle *> (item);
+		if (rect) {
+			rect->set_fill_color (UIConfiguration::instance().color ("inactive fade handle"));
+		}
+	}
+	break;
+
+	case AutomationTrackItem:
+		break;
+
+	case FeatureLineItem:
+	{
+		ArdourCanvas::Line *line = dynamic_cast<ArdourCanvas::Line *> (item);
+		line->set_outline_color (UIConfiguration::instance().color ("zero line"));
+	}
+	break;
+
+	default:
+		_region_peak_cursor->hide ();
+		break;
 	}
 
-	return _visible_canvas_height - _trackview_group->canvas_origin().y;
+	return ret;
 }
