@@ -24,31 +24,37 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
+#include <cstdint>
 #include <cstdlib>
 #include <cmath>
 #include <algorithm>
+#include <memory>
 #include <typeinfo>
 
+#include "ardour/audioregion.h"
 #include "ardour/audio_track.h"
 #include "ardour/midi_track.h"
 #include "ardour/midi_region.h"
+#include "ardour/operations.h"
 #include "ardour/profile.h"
 #include "ardour/region_factory.h"
 
+#include "ardour/types.h"
 #include "canvas/canvas.h"
 #include "canvas/text.h"
 #include "canvas/scroll_group.h"
 
 #include "editor.h"
+#include "editor_sections.h"
 #include "keyboard.h"
 #include "public_editor.h"
 #include "audio_region_view.h"
 #include "audio_streamview.h"
 #include "audio_time_axis.h"
 #include "region_gain_line.h"
-#include "automation_line.h"
+#include "editor_automation_line.h"
 #include "automation_time_axis.h"
-#include "automation_line.h"
+#include "editor_automation_line.h"
 #include "control_point.h"
 #include "editor_drag.h"
 #include "midi_time_axis.h"
@@ -151,7 +157,7 @@ bool
 Editor::canvas_scroll_event (GdkEventScroll *event, bool from_canvas)
 {
 	if (from_canvas) {
-		boost::optional<ArdourCanvas::Rect> rulers = _time_markers_group->bounding_box();
+		std::optional<ArdourCanvas::Rect> rulers = _time_markers_group->bounding_box();
 		if (rulers && rulers->contains (Duple (event->x, event->y))) {
 			return canvas_ruler_event ((GdkEvent*) event, timecode_ruler, TimecodeRulerItem);
 		}
@@ -201,50 +207,6 @@ Editor::track_canvas_motion_notify_event (GdkEventMotion */*event*/)
 	return false;
 }
 
-bool
-Editor::typed_event (ArdourCanvas::Item* item, GdkEvent *event, ItemType type)
-{
-	if (!session () || session()->loading () || session()->deletion_in_progress ()) {
-		return false;
-	}
-
-	gint ret = FALSE;
-
-	switch (event->type) {
-	case GDK_BUTTON_PRESS:
-	case GDK_2BUTTON_PRESS:
-	case GDK_3BUTTON_PRESS:
-		ret = button_press_handler (item, event, type);
-		break;
-	case GDK_BUTTON_RELEASE:
-		ret = button_release_handler (item, event, type);
-		break;
-	case GDK_MOTION_NOTIFY:
-		ret = motion_handler (item, event);
-		break;
-
-	case GDK_ENTER_NOTIFY:
-		ret = enter_handler (item, event, type);
-		break;
-
-	case GDK_LEAVE_NOTIFY:
-		ret = leave_handler (item, event, type);
-		break;
-
-	case GDK_KEY_PRESS:
-		ret = key_press_handler (item, event, type);
-		break;
-
-	case GDK_KEY_RELEASE:
-		ret = key_release_handler (item, event, type);
-		break;
-
-	default:
-		break;
-	}
-	return ret;
-}
-
 bool
 Editor::canvas_region_view_event (GdkEvent *event, ArdourCanvas::Item* item, RegionView *rv)
 {
@@ -374,6 +336,13 @@ Editor::canvas_stream_view_event (GdkEvent *event, ArdourCanvas::Item* item, Rou
 bool
 Editor::canvas_automation_track_event (GdkEvent *event, ArdourCanvas::Item* item, AutomationTimeAxisView *atv)
 {
+	if (atv->parameter().type() == MidiVelocityAutomation) {
+		/* no event handling for velocity tracks until we can make the
+		   automation control affect note velocity.
+		*/
+		return false;
+	}
+
 	bool ret = false;
 
 	switch (event->type) {
@@ -403,6 +372,18 @@ Editor::canvas_automation_track_event (GdkEvent *event, ArdourCanvas::Item* item
 		ret = leave_handler (item, event, AutomationTrackItem);
 		break;
 
+	case GDK_KEY_PRESS:
+		if (_drags->active()) {
+			return _drags->mid_drag_key_event (&event->key);
+		}
+		break;
+
+	case GDK_KEY_RELEASE:
+		if (_drags->active()) {
+			return _drags->mid_drag_key_event (&event->key);
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -660,7 +641,7 @@ Editor::canvas_control_point_event (GdkEvent *event, ArdourCanvas::Item* item, C
 	case GDK_2BUTTON_PRESS:
 	case GDK_3BUTTON_PRESS:
 		clicked_control_point = cp;
-		clicked_axisview = &cp->line().trackview;
+		clicked_axisview = &dynamic_cast<EditorAutomationLine*> (&cp->line())->trackview;
 		clicked_routeview = dynamic_cast<RouteTimeAxisView*>(clicked_axisview);
 		clicked_regionview = 0;
 		break;
@@ -672,17 +653,29 @@ Editor::canvas_control_point_event (GdkEvent *event, ArdourCanvas::Item* item, C
 }
 
 bool
-Editor::canvas_line_event (GdkEvent *event, ArdourCanvas::Item* item, AutomationLine* al)
+Editor::canvas_velocity_event (GdkEvent *event, ArdourCanvas::Item* item)
+{
+	return typed_event (item, event, VelocityItem);
+}
+
+bool
+Editor::canvas_velocity_base_event (GdkEvent *event, ArdourCanvas::Item* item)
+{
+	return typed_event (item, event, VelocityBaseItem);
+}
+
+bool
+Editor::canvas_line_event (GdkEvent *event, ArdourCanvas::Item* item, EditorAutomationLine* al)
 {
 	ItemType type;
-	AudioRegionGainLine* gl;
-	if ((gl = dynamic_cast<AudioRegionGainLine*> (al)) != 0) {
+	RegionFxLine* rfl;
+	if ((rfl = dynamic_cast<RegionFxLine*> (al)) != 0) {
 		type = GainLineItem;
 		if (event->type == GDK_BUTTON_PRESS) {
-			clicked_regionview = &gl->region_view ();
+			clicked_regionview = &rfl->region_view ();
 		}
 	} else {
-		type = AutomationLineItem;
+		type = EditorAutomationLineItem;
 		if (event->type == GDK_BUTTON_PRESS) {
 			clicked_regionview = 0;
 		}
@@ -837,6 +830,7 @@ Editor::canvas_frame_handle_event (GdkEvent* event, ArdourCanvas::Item* item, Re
 		ret = motion_handler (item, event);
 		break;
 	case GDK_ENTER_NOTIFY:
+		set_entered_regionview (rv);
 		ret = enter_handler (item, event, type);
 		break;
 
@@ -1095,12 +1089,112 @@ Editor::canvas_ruler_bar_event (GdkEvent *event, ArdourCanvas::Item* item, ItemT
 	return typed_event (item, event, type);
 }
 
+bool
+Editor::section_rect_event (GdkEvent* ev, Location* loc, ArdourCanvas::Rectangle* rect, std::string color)
+{
+	switch (ev->type) {
+		case GDK_ENTER_NOTIFY:
+			if (UIConfiguration::instance ().get_widget_prelight ()) {
+				rect->set_fill_color (Gtkmm2ext::HSV (UIConfiguration::instance().color (color)).lighter (0.25));
+				return true;
+			}
+			break;
+		case GDK_LEAVE_NOTIFY:
+			if (UIConfiguration::instance ().get_widget_prelight ()) {
+				rect->set_fill_color (UIConfiguration::instance().color (color));
+				return true;
+			}
+			break;
+		case GDK_BUTTON_PRESS:
+			if (Keyboard::modifier_state_equals (ev->button.state, Keyboard::PrimaryModifier)) {
+				/* used to add markers */
+				return false;
+			}
+			if (ev->button.button == 1) {
+				_session->request_locate (loc->start().samples());
+			}
+			return true;
+		case GDK_2BUTTON_PRESS:
+		case GDK_3BUTTON_PRESS:
+			if (Keyboard::modifier_state_equals (ev->button.state, Keyboard::PrimaryModifier)) {
+				return false;
+			}
+			if (ev->button.button == 1) {
+				assert (find_location_markers (loc));
+				edit_marker (find_location_markers (loc)->start, true);
+				return true;
+			}
+			break;
+		case GDK_BUTTON_RELEASE:
+			if (Keyboard::is_context_menu_event (&ev->button)) {
+				using namespace Menu_Helpers;
+
+				/* find section */
+				timepos_t start (loc->start ());
+				timepos_t end;
+				Location* l = _session->locations()->section_at (start, start, end);
+				assert (l);
+
+				timepos_t where (canvas_event_time (ev));
+				snap_to (where, Temporal::RoundNearest);
+
+				section_box_menu.set_name ("ArdourContextMenu");
+				MenuList& items (section_box_menu.items());
+				items.clear ();
+
+				items.push_back (MenuElem (_("New Arrangement Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark | Location::IsSection), 0)));
+				items.push_back (MenuElem (_("Select Arrangement Section"), sigc::bind (sigc::mem_fun(*_sections, &EditorSections::select), l)));
+#if 0
+				items.push_back (SeparatorElem());
+				add_section_context_items (items);   //TODO: section_context_items needs to be modified to operate on the marker you clicked on, not the range selection (which might not exist)
+#endif
+
+				section_box_menu.popup (ev->button.button, ev->button.time);
+				return true;
+			}
+			break;
+		default:
+			break;
+	}
+	return false;
+}
+
 bool
 Editor::canvas_playhead_cursor_event (GdkEvent *event, ArdourCanvas::Item* item)
 {
 	return typed_event (item, event, PlayheadCursorItem);
 }
 
+bool
+Editor::canvas_section_box_event (GdkEvent *event)
+{
+	switch (event->type) {
+		case GDK_BUTTON_PRESS:
+			if (!Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier)
+			   && event->button.button == 1) {
+				_drags->set (new CursorDrag (*this, *_playhead_cursor, false), event);
+			}
+			/*fallthrough*/
+		case GDK_2BUTTON_PRESS:
+			/*fallthrough*/
+		case GDK_3BUTTON_PRESS:
+			return !Keyboard::modifier_state_equals (event->button.state, Keyboard::PrimaryModifier);
+		case GDK_BUTTON_RELEASE:
+			if (Keyboard::is_context_menu_event (&event->button)) {
+				section_box_menu.set_name ("ArdourContextMenu");
+				Gtk::Menu_Helpers::MenuList& items (section_box_menu.items());
+				items.clear ();
+				add_section_context_items (items);
+				section_box_menu.popup (event->button.button, event->button.time);
+				return true;
+			}
+			return false;
+		default:
+			break;
+	}
+	return false;
+}
+
 bool
 Editor::canvas_note_event (GdkEvent *event, ArdourCanvas::Item* item)
 {
@@ -1111,6 +1205,16 @@ Editor::canvas_note_event (GdkEvent *event, ArdourCanvas::Item* item)
 	return typed_event (item, event, NoteItem);
 }
 
+bool
+Editor::canvas_bg_event (GdkEvent *event, ArdourCanvas::Item* item)
+{
+	if (!internal_editing()) {
+		return false;
+	}
+
+	return typed_event (item, event, RegionItem);
+}
+
 bool
 Editor::canvas_drop_zone_event (GdkEvent* event)
 {
@@ -1129,8 +1233,8 @@ Editor::canvas_drop_zone_event (GdkEvent* event)
 
 	case GDK_SCROLL:
 		/* convert coordinates back into window space so that
-		   we can just call canvas_scroll_event().
-		*/
+		 * we can just call canvas_scroll_event().
+		 */
 		winpos = _track_canvas->canvas_to_window (Duple (event->scroll.x, event->scroll.y));
 		scroll = event->scroll;
 		scroll.x = winpos.x;
@@ -1155,6 +1259,53 @@ Editor::canvas_drop_zone_event (GdkEvent* event)
 	return true;
 }
 
+bool
+Editor::canvas_grid_zone_event (GdkEvent* event)
+{
+	GdkEventScroll scroll;
+	ArdourCanvas::Duple winpos;
+
+
+	switch (event->type) {
+
+		case GDK_BUTTON_PRESS:
+			if (event->button.button == 1) {
+				choose_mapping_drag (_canvas_grid_zone, event);
+			}
+			break;
+
+		case GDK_BUTTON_RELEASE:
+			return typed_event (_canvas_grid_zone, event, GridZoneItem);
+			break;
+
+		case GDK_SCROLL:
+			/* convert coordinates back into window space so that
+			 * we can just call canvas_scroll_event().
+			 */
+			winpos   = _track_canvas->canvas_to_window (Duple (event->scroll.x, event->scroll.y));
+			scroll   = event->scroll;
+			scroll.x = winpos.x;
+			scroll.y = winpos.y;
+			return canvas_scroll_event (&scroll, true);
+			break;
+
+		case GDK_ENTER_NOTIFY:
+			return typed_event (_canvas_grid_zone, event, GridZoneItem);
+
+		case GDK_LEAVE_NOTIFY:
+			return typed_event (_canvas_grid_zone, event, GridZoneItem);
+
+		case GDK_MOTION_NOTIFY:
+			return motion_handler (_canvas_grid_zone, event);
+			break;
+
+		default:
+			break;
+	}
+
+	return true;
+}
+
 bool
 Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context, int x, int y, guint time)
 {
@@ -1201,7 +1352,7 @@ Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context,
 
 	if (can_drop) {
 
-		if (target == "x-ardour/region.pbdid") {
+		if (target == "x-ardour/region.pbdid" || target == "x-ardour/region.pbdids") {
 			if (tv.first == 0 && pbdid_dragged_dt != DataType::NIL) {
 				/* drop to drop-zone */
 				context->drag_status (Gdk::ACTION_COPY, time);
@@ -1239,23 +1390,22 @@ Editor::track_canvas_drag_motion (Glib::RefPtr<Gdk::DragContext> const& context,
 }
 
 void
-Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
+Editor::drop_region (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
                       int x, int y,
-                      const SelectionData& data,
+                      PBD::ID const& rid,
                       guint /*info*/, guint /*time*/)
 {
 	GdkEvent event;
 	double px;
 	double py;
 
-	event.type = GDK_MOTION_NOTIFY;
+	event.type = GDK_BUTTON_PRESS;
 	event.button.x = x;
 	event.button.y = y;
 	/* assume we're dragging with button 1 */
 	event.motion.state = Gdk::BUTTON1_MASK;
 	samplepos_t const pos = window_event_sample (&event, &px, &py);
 
-	PBD::ID rid (data.get_data_as_string ());
 	std::shared_ptr<Region> region = RegionFactory::region_by_id (rid);
 
 	if (!region) { return; }
@@ -1272,7 +1422,7 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 				if ((Config->get_output_auto_connect() & AutoConnectMaster) && session()->master_out()) {
 					output_chan =  session()->master_out()->n_inputs().n_audio();
 				}
-				list<std::shared_ptr<AudioTrack> > audio_tracks;
+				AudioTrackList audio_tracks;
 				audio_tracks = session()->new_audio_track (region->sources().size(), output_chan, 0, 1, region->name(), PresentationInfo::max_order);
 				rtav = dynamic_cast<RouteTimeAxisView*> (time_axis_view_from_stripable (audio_tracks.front()));
 			} else if (std::dynamic_pointer_cast<MidiRegion> (region)) {
@@ -1282,7 +1432,7 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 				                                         Config->get_strict_io () || Profile->get_mixbus (),
 				                                         std::shared_ptr<ARDOUR::PluginInfo>(),
 				                                         (ARDOUR::Plugin::PresetRecord*) 0,
-				                                         (ARDOUR::RouteGroup*) 0, 1, region->name(), PresentationInfo::max_order, Normal, true);
+				                                         nullptr, 1, region->name(), PresentationInfo::max_order, Normal, true);
 				rtav = dynamic_cast<RouteTimeAxisView*> (time_axis_view_from_stripable (midi_tracks.front()));
 			} else {
 				return;
@@ -1298,12 +1448,192 @@ Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& /*context*/,
 
 		if ((std::dynamic_pointer_cast<AudioRegion> (region_copy) != 0 && dynamic_cast<AudioTimeAxisView*> (rtav) != 0) ||
 		    (std::dynamic_pointer_cast<MidiRegion> (region_copy) != 0 && dynamic_cast<MidiTimeAxisView*> (rtav) != 0)) {
-			_drags->set (new RegionInsertDrag (this, region_copy, rtav, timepos_t (pos), drag_time_domain (region_copy.get())), &event);
+			_drags->set (new RegionInsertDrag (*this, region_copy, rtav, timepos_t (pos), drag_time_domain (region_copy.get())), &event);
 			_drags->end_grab (&event);
 		}
 	}
 }
 
+void
+Editor::drop_regions (const Glib::RefPtr<Gdk::DragContext>& context,
+                      int x, int y,
+                      std::vector<PBD::ID> const& rids,
+                      guint info, guint time)
+{
+	if (rids.empty ()) {
+		return;
+	}
+
+	GdkEvent event;
+	double px;
+	double py;
+
+	event.type = GDK_BUTTON_PRESS;
+	event.button.x = x;
+	event.button.y = y;
+	/* assume we're dragging with button 1 */
+	event.motion.state = Gdk::BUTTON1_MASK;
+	samplepos_t const pos = window_event_sample (&event, &px, &py);
+
+	std::pair<TimeAxisView*, int> const tv = trackview_by_y_position (py, false);
+
+	if (tv.first == 0) {
+		/* drop to dropzone */
+		std::map<shared_ptr<Playlist>, shared_ptr<Region>> import_map;
+
+		/* step 1: create tracks */
+		if (pbdid_dragged_dt == DataType::MIDI) {
+			for (auto const& rid : rids) {
+				std::shared_ptr<Region> region = RegionFactory::region_by_id (rid);
+				ChanCount one_midi_port (DataType::MIDI, 1);
+				list<std::shared_ptr<MidiTrack> > midi_tracks;
+				midi_tracks = session()->new_midi_track (one_midi_port, one_midi_port,
+				                                         Config->get_strict_io () || Profile->get_mixbus (),
+				                                         std::shared_ptr<ARDOUR::PluginInfo>(),
+				                                         (ARDOUR::Plugin::PresetRecord*) 0,
+				                                         nullptr, 1, region->name(), PresentationInfo::max_order, Normal, true);
+				if (midi_tracks.size () == 1) {
+					std::shared_ptr<Playlist> playlist = midi_tracks.front()->playlist ();
+					std::shared_ptr<Region> region_copy = RegionFactory::create (region, true);
+					import_map[playlist] = region_copy;
+				}
+			}
+		} else if (pbdid_dragged_dt == DataType::AUDIO) {
+			RouteList routes;
+			AudioTrackList tracks;
+			bool use_master_chan = (Config->get_output_auto_connect() & AutoConnectMaster) && session()->master_out();
+
+			for (auto const& rid : rids) {
+				std::shared_ptr<Region> region = RegionFactory::region_by_id (rid);
+				uint32_t output_chan = region->sources().size();
+				if (use_master_chan) {
+					output_chan = _session->master_out()->n_inputs().n_audio();
+				}
+				bool ok = _session->new_audio_routes_tracks_bulk (routes, tracks, region->sources().size(), output_chan, 0, 1, region->name(), PresentationInfo::max_order);
+				if (ok) {
+					std::shared_ptr<Playlist> playlist = tracks.back()->playlist ();
+					std::shared_ptr<Region> region_copy = RegionFactory::create (region, true);
+					import_map[playlist] = region_copy;
+				}
+
+			}
+			if (!routes.empty ()) {
+				_session->add_routes (routes, true, true, PresentationInfo::max_order);
+			}
+		}
+
+		/* step 2: add regions to created tracks */
+		timepos_t insert_pos (pos);
+		snap_to_with_modifier (insert_pos, &event);
+		begin_reversible_command (Operations::insert_region);
+
+		for (auto& [playlist, r] : import_map) {
+			playlist->clear_changes ();
+			playlist->clear_owned_changes ();
+			playlist->add_region (r, insert_pos, 1.0, false);
+			playlist->rdiff_and_add_command (_session);
+		}
+		commit_reversible_command ();
+
+		return;
+	}
+
+	RouteTimeAxisView* rtav = dynamic_cast<RouteTimeAxisView*> (tv.first);
+	if (!rtav) {
+		return;
+	}
+
+	bool sequence_regions;
+	std::shared_ptr<AudioRegion> first_region;
+
+	if (pbdid_dragged_dt == DataType::MIDI && 0 != dynamic_cast<MidiTimeAxisView*> (rtav)) {
+		sequence_regions = true;
+	} else if (pbdid_dragged_dt == DataType::AUDIO && 0 != dynamic_cast<AudioTimeAxisView*> (rtav)) {
+		uint32_t chn = 0;
+		size_t   nar = 0;
+		timecnt_t length;
+
+		for (auto const& rid : rids) {
+			std::shared_ptr<AudioRegion> ar = dynamic_pointer_cast<AudioRegion> (RegionFactory::region_by_id (rid));
+			if (!ar) {
+				continue;
+			}
+			chn += ar->n_channels ();
+			if (length.is_zero ()) {
+				first_region = ar;
+				length = ar->length ();
+			} else if (length != ar->length ()) {
+				chn = UINT32_MAX;
+				break;
+			}
+			++nar;
+		}
+
+		// XXX check route's _disk_reader->output_streams().n_audio() ?
+		if (rtav->route ()->n_inputs ().n_audio () != chn || chn < 2 || nar < 2 || chn == UINT32_MAX) {
+			sequence_regions = true;
+		} else {
+			sequence_regions = false;
+		}
+	} else {
+		assert (0);
+		return;
+	}
+
+	if (sequence_regions) {
+		timepos_t insert_pos (pos);
+		snap_to_with_modifier (insert_pos, &event);
+
+		std::shared_ptr<Playlist> playlist = rtav->playlist ();
+		begin_reversible_command (Operations::insert_region);
+		playlist->clear_changes ();
+		playlist->clear_owned_changes ();
+
+		for (auto const& rid : rids) {
+			std::shared_ptr<Region> region = RegionFactory::region_by_id (rid);
+			std::shared_ptr<Region> region_copy = RegionFactory::create (region, true);
+			playlist->add_region (region_copy, insert_pos, 1.0, false);
+			if (should_ripple ()) {
+				playlist->ripple (insert_pos, region_copy->length (), region_copy);
+			}
+			insert_pos += region_copy->length ();
+		}
+		playlist->rdiff_and_add_command (_session);
+		commit_reversible_command ();
+	} else {
+		/* make multi-channel region */
+		std::string region_name = region_name_from_path (first_region->name(), true, false); // XXX use track name?
+		
+		while (RegionFactory::region_by_name (region_name)) {
+			region_name = bump_name_once (region_name, '.');
+		}
+
+		PropertyList plist;
+		plist.add (ARDOUR::Properties::start, timecnt_t (Temporal::AudioTime));
+		plist.add (ARDOUR::Properties::length, first_region->length ());
+		plist.add (ARDOUR::Properties::name, region_name);
+		plist.add (ARDOUR::Properties::layer, 0);
+		plist.add (ARDOUR::Properties::whole_file, false);
+		plist.add (ARDOUR::Properties::external, true);
+		plist.add (ARDOUR::Properties::opaque, true);
+
+		SourceList sources;
+		for (auto const& rid : rids) {
+			std::shared_ptr<AudioRegion> ar = dynamic_pointer_cast<AudioRegion> (RegionFactory::region_by_id (rid));
+			if (!ar) {
+				continue;
+			}
+			sources.insert (sources.end (), ar->sources ().begin(), ar->sources ().end());
+		}
+
+		std::shared_ptr<Region> r = RegionFactory::create (sources, plist);
+		std::dynamic_pointer_cast<AudioRegion>(r)->special_set_position (sources[0]->natural_position());
+
+		_drags->set (new RegionInsertDrag (*this, r, rtav, timepos_t (pos), Temporal::AudioTime), &event);
+		_drags->end_grab (&event);
+	}
+}
+
 bool
 Editor::key_press_handler (ArdourCanvas::Item*, GdkEvent*, ItemType)
 {
