@@ -33,7 +33,7 @@
 
 #include <string>
 
-#include <gtk/gtkaction.h>
+#include <ytk/gtkaction.h>
 
 #include "canvas/container.h"
 #include "canvas/canvas.h"
@@ -134,8 +134,13 @@ Editor::initialize_rulers ()
 {
 	ruler_grabbed_widget = 0;
 
-	Pango::FontDescription font (UIConfiguration::instance().get_SmallerFont());
-	Pango::FontDescription larger_font (UIConfiguration::instance().get_SmallBoldFont());
+#ifdef __APPLE__
+	Pango::FontDescription font (UIConfiguration::instance().get_VerySmallFont());
+	Pango::FontDescription bold_font (UIConfiguration::instance().get_VerySmallBoldFont());
+#else
+	Pango::FontDescription font (UIConfiguration::instance().get_SmallFont());
+	Pango::FontDescription bold_font (UIConfiguration::instance().get_SmallBoldFont());
+#endif
 
 	_timecode_metric = new TimecodeMetric (this);
 	_bbt_metric = new BBTMetric (this);
@@ -144,41 +149,44 @@ Editor::initialize_rulers ()
 
 	timecode_ruler = new ArdourCanvas::Ruler (_time_markers_group, _timecode_metric,
 						  ArdourCanvas::Rect (0, 0, ArdourCanvas::COORD_MAX, timebar_height));
-	timecode_ruler->set_font_description (font);
+	timecode_ruler->set_font_description (bold_font);
+	timecode_ruler->set_minor_font_description (font);
 	CANVAS_DEBUG_NAME (timecode_ruler, "timecode ruler");
 	timecode_nmarks = 0;
 
 	samples_ruler = new ArdourCanvas::Ruler (_time_markers_group, _samples_metric,
 						 ArdourCanvas::Rect (0, 0, ArdourCanvas::COORD_MAX, timebar_height));
-	samples_ruler->set_font_description (font);
+	samples_ruler->set_font_description (bold_font);
+	samples_ruler->set_minor_font_description (font);
 	CANVAS_DEBUG_NAME (samples_ruler, "samples ruler");
 
 	minsec_ruler = new ArdourCanvas::Ruler (_time_markers_group, _minsec_metric,
 						ArdourCanvas::Rect (0, 0, ArdourCanvas::COORD_MAX, timebar_height));
-	minsec_ruler->set_font_description (font);
+	minsec_ruler->set_font_description (bold_font);
+	minsec_ruler->set_minor_font_description (font);
 	CANVAS_DEBUG_NAME (minsec_ruler, "minsec ruler");
 	minsec_nmarks = 0;
 
 	bbt_ruler = new ArdourCanvas::Ruler (_time_markers_group, _bbt_metric,
 	                                     ArdourCanvas::Rect (0, 0, ArdourCanvas::COORD_MAX, timebar_height));
-	bbt_ruler->set_font_description (font);
-	bbt_ruler->set_second_font_description (larger_font);
+	bbt_ruler->set_font_description (bold_font);
+	bbt_ruler->set_minor_font_description (font);
 	CANVAS_DEBUG_NAME (bbt_ruler, "bbt ruler");
 	timecode_nmarks = 0;
 
 	using namespace Box_Helpers;
 	BoxList & lab_children =  time_bars_vbox.children();
 
-	lab_children.push_back (Element(minsec_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(timecode_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(samples_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(bbt_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(tempo_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(meter_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(range_mark_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(mark_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(section_mark_label, PACK_SHRINK, PACK_START));
-	lab_children.push_back (Element(videotl_label, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_minsec, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_timecode, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_samples, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_bbt, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_tempo, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_meter, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_range, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_marker, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_section, PACK_SHRINK, PACK_START));
+	lab_children.push_back (Element(_ruler_box_videotl, PACK_SHRINK, PACK_START));
 
 	/* 1 event handler to bind them all ... */
 
@@ -190,6 +198,25 @@ Editor::initialize_rulers ()
 	visible_timebars = 0; /*this will be changed below */
 }
 
+void
+Editor::initialize_ruler_actions ()
+{
+	_ruler_btn_loc_prev.set_related_action (ActionManager::get_action (X_("Common"), X_("jump-backward-to-location-mark")));
+	_ruler_btn_loc_next.set_related_action (ActionManager::get_action (X_("Common"), X_("jump-forward-to-location-mark")));
+	_ruler_btn_loc_add.set_related_action (ActionManager::get_action (X_("Common"), X_("add-location-from-playhead")));
+
+	_ruler_btn_section_prev.set_related_action (ActionManager::get_action (X_("Common"), X_("jump-backward-to-section-mark")));
+	_ruler_btn_section_next.set_related_action (ActionManager::get_action (X_("Common"), X_("jump-forward-to-section-mark")));
+	_ruler_btn_section_add.set_related_action (ActionManager::get_action (X_("Common"), X_("add-section-from-playhead")));
+
+	_ruler_btn_range_add.set_related_action (ActionManager::get_action (X_("Editor"), X_("add-range-marker-from-selection")));
+	_ruler_btn_range_prev.set_related_action (ActionManager::get_action (X_("Common"), X_("jump-backward-to-range")));
+	_ruler_btn_range_next.set_related_action (ActionManager::get_action (X_("Common"), X_("jump-forward-to-range")));
+
+	_ruler_btn_tempo_add.set_related_action (ActionManager::get_action (X_("Editor"), X_("add-tempo-from-playhead")));
+	_ruler_btn_meter_add.set_related_action (ActionManager::get_action (X_("Editor"), X_("add-meter-from-playhead")));
+}
+
 bool
 Editor::ruler_label_button_release (GdkEventButton* ev)
 {
@@ -244,11 +271,11 @@ Editor::popup_ruler_menu (timepos_t const & where, ItemType t)
 		 */
 		//ruler_items.push_back (MenuElem (_("Timeline height"))); // heading
 		//static_cast<MenuItem*>(&ruler_items.back())->set_sensitive(false);
-		ruler_items.push_back (CheckMenuElem (_("Large"),  sigc::bind (sigc::mem_fun(*this, &Editor::set_video_timeline_height), 6)));
+		ruler_items.push_back (CheckMenuElem (_("Large"),  sigc::bind (sigc::mem_fun(*this, &Editor::set_video_timeline_height), 6, false)));
 		if (videotl_bar_height == 6) { static_cast<Gtk::CheckMenuItem*>(&ruler_items.back())->set_active(true);}
-		ruler_items.push_back (CheckMenuElem (_("Normal"), sigc::bind (sigc::mem_fun(*this, &Editor::set_video_timeline_height), 4)));
+		ruler_items.push_back (CheckMenuElem (_("Normal"), sigc::bind (sigc::mem_fun(*this, &Editor::set_video_timeline_height), 4, false)));
 		if (videotl_bar_height == 4) { static_cast<Gtk::CheckMenuItem*>(&ruler_items.back())->set_active(true);}
-		ruler_items.push_back (CheckMenuElem (_("Small"),  sigc::bind (sigc::mem_fun(*this, &Editor::set_video_timeline_height), 3)));
+		ruler_items.push_back (CheckMenuElem (_("Small"),  sigc::bind (sigc::mem_fun(*this, &Editor::set_video_timeline_height), 3, false)));
 		if (videotl_bar_height == 3) { static_cast<Gtk::CheckMenuItem*>(&ruler_items.back())->set_active(true);}
 
 		ruler_items.push_back (SeparatorElem ());
@@ -289,7 +316,7 @@ Editor::popup_ruler_menu (timepos_t const & where, ItemType t)
 		add_items.push_back (MenuElem (_("Location Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags (Location::IsMark), 0)));
 		add_items.push_back (MenuElem (_("Arrangement Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark | Location::IsSection), 0)));
 		add_items.push_back (MenuElem (_("CD Track Marker"), sigc::bind (sigc::mem_fun(*this, &Editor::add_location_mark_with_flag), where, Location::Flags(Location::IsMark |Location::IsCDMarker), 0)));
-		add_items.push_back (MenuElem ("Cue Marker..."));
+		add_items.push_back (MenuElem (_("Cue Marker...")));
 		Gtk::MenuItem& cue_submenu = add_items.back();
 		Gtk::Menu* cue_menu = new Gtk::Menu;
 		MenuList& cue_items = cue_menu->items();
@@ -466,18 +493,16 @@ Editor::update_ruler_visibility ()
 	 */
 
 	double tbpos = 0.0;
-	double tbgpos = 0.0;
 	double old_unit_pos;
 
 #ifdef __APPLE__
 	/* gtk update probs require this (damn) */
-	meter_label.hide();
-	tempo_label.hide();
-	range_mark_label.hide();
-	section_mark_label.hide();
-	cue_mark_label.hide();
-	mark_label.hide();
-	videotl_label.hide();
+	_ruler_box_tempo.hide();
+	_ruler_box_meter.hide();
+	_ruler_box_range.hide();
+	_ruler_box_marker.hide();
+	_ruler_box_section.hide();
+	_ruler_box_videotl.hide();
 #endif
 
 	if (ruler_minsec_action->get_active()) {
@@ -486,14 +511,14 @@ Editor::update_ruler_visibility ()
 			minsec_ruler->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		minsec_ruler->show();
-		minsec_label.show();
+		_ruler_box_minsec.show();
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
+
 		visible_timebars++;
 		have_timebar = true;
 	} else {
 		minsec_ruler->hide();
-		minsec_label.hide();
+		_ruler_box_minsec.hide();
 	}
 
 	if (ruler_timecode_action->get_active()) {
@@ -502,14 +527,13 @@ Editor::update_ruler_visibility ()
 			timecode_ruler->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		timecode_ruler->show();
-		timecode_label.show();
+		_ruler_box_timecode.show();
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 		have_timebar = true;
 	} else {
 		timecode_ruler->hide();
-		timecode_label.hide();
+		_ruler_box_timecode.hide();
 	}
 
 	if (ruler_samples_action->get_active()) {
@@ -518,14 +542,13 @@ Editor::update_ruler_visibility ()
 			samples_ruler->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		samples_ruler->show();
-		samples_label.show();
+		_ruler_box_samples.show();
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 		have_timebar = true;
 	} else {
 		samples_ruler->hide();
-		samples_label.hide();
+		_ruler_box_samples.hide();
 	}
 
 	if (ruler_bbt_action->get_active()) {
@@ -534,14 +557,13 @@ Editor::update_ruler_visibility ()
 			bbt_ruler->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		bbt_ruler->show();
-		bbt_label.show();
+		_ruler_box_bbt.show();
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 		have_timebar = true;
 	} else {
 		bbt_ruler->hide();
-		bbt_label.hide();
+		_ruler_box_bbt.hide();
 	}
 
 	if (ruler_tempo_action->get_active()) {
@@ -550,13 +572,12 @@ Editor::update_ruler_visibility ()
 			tempo_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		tempo_group->show();
-		tempo_label.show();
+		_ruler_box_tempo.show();
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 	} else {
 		tempo_group->hide();
-		tempo_label.hide();
+		_ruler_box_tempo.hide();
 	}
 
 	if (ruler_meter_action->get_active()) {
@@ -565,13 +586,12 @@ Editor::update_ruler_visibility ()
 			meter_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		meter_group->show();
-		meter_label.show();
+		_ruler_box_meter.show();
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 	} else {
 		meter_group->hide();
-		meter_label.hide();
+		_ruler_box_meter.hide();
 	}
 
 	if (ruler_range_action->get_active()) {
@@ -580,16 +600,15 @@ Editor::update_ruler_visibility ()
 			range_marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		range_marker_group->show();
-		range_mark_label.show();
+		_ruler_box_range.show();
 
-		range_marker_bar->set_outline(false);
+		range_marker_bar->set_outline (true);
 
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 	} else {
 		range_marker_group->hide();
-		range_mark_label.hide();
+		_ruler_box_range.hide();
 	}
 
 	if (ruler_marker_action->get_active()) {
@@ -598,16 +617,15 @@ Editor::update_ruler_visibility ()
 			marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		marker_group->show();
-		mark_label.show();
+		_ruler_box_marker.show();
 
-		marker_bar->set_outline(false);
+		marker_bar->set_outline (true);
 
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 	} else {
 		marker_group->hide();
-		mark_label.hide();
+		_ruler_box_marker.hide();
 	}
 
 	if (!Profile->get_livetrax() && ruler_section_action->get_active()) {
@@ -616,17 +634,16 @@ Editor::update_ruler_visibility ()
 			section_marker_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		section_marker_group->show();
-		section_mark_label.show();
+		_ruler_box_section.show();
 
-		section_marker_bar->set_outline(false);
+		section_marker_bar->set_outline (true);
 
 		tbpos += timebar_height;
-		tbgpos += timebar_height;
 		visible_timebars++;
 		update_marker_display();
 	} else {
 		section_marker_group->hide();
-		section_mark_label.hide();
+		_ruler_box_section.hide();
 		update_marker_display();
 	}
 
@@ -636,14 +653,13 @@ Editor::update_ruler_visibility ()
 			videotl_group->move (ArdourCanvas::Duple (0.0, tbpos - old_unit_pos));
 		}
 		videotl_group->show();
-		videotl_label.show();
+		_ruler_box_videotl.show();
 		tbpos += timebar_height * videotl_bar_height;
-		tbgpos += timebar_height * videotl_bar_height;
 		visible_timebars+=videotl_bar_height;
 		queue_visual_videotimeline_update();
 	} else {
 		videotl_group->hide();
-		videotl_label.hide();
+		_ruler_box_videotl.hide();
 		update_video_timeline(true);
 	}
 
@@ -1045,111 +1061,6 @@ Editor::metric_get_timecode (std::vector<ArdourCanvas::Ruler::Mark>& marks, int6
 	}
 }
 
-uint32_t
-Editor::count_bars (Beats const & start, Beats const & end) const
-{
-	TempoMapPoints bar_grid;
-	TempoMap::SharedPtr tmap (TempoMap::use());
-	bar_grid.reserve (4096);
-	superclock_t s (tmap->superclock_at (start));
-	superclock_t e (tmap->superclock_at (end));
-	tmap->get_grid (bar_grid, s, e, 1);
-	return bar_grid.size();
-}
-
-void
-Editor::compute_bbt_ruler_scale (samplepos_t lower, samplepos_t upper)
-{
-	if (_session == 0) {
-		return;
-	}
-
-	Temporal::BBT_Time lower_beat, upper_beat; // the beats at each end of the ruler
-	Temporal::TempoMap::SharedPtr tmap (Temporal::TempoMap::use());
-	Beats floor_lower_beat = std::max (Beats(), tmap->quarters_at_sample (lower)).round_down_to_beat ();
-
-	if (floor_lower_beat < Temporal::Beats()) {
-		floor_lower_beat = Temporal::Beats();
-	}
-
-	const samplepos_t beat_before_lower_pos = tmap->sample_at (floor_lower_beat);
-	const samplepos_t beat_after_upper_pos = tmap->sample_at ((std::max (Beats(), tmap->quarters_at_sample  (upper)).round_down_to_beat()) + Beats (1, 0));
-
-	lower_beat = Temporal::TempoMap::use()->bbt_at (timepos_t (beat_before_lower_pos));
-	upper_beat = Temporal::TempoMap::use()->bbt_at (timepos_t (beat_after_upper_pos));
-	uint32_t beats = 0;
-
-	bbt_bar_helper_on = false;
-	bbt_bars = 0;
-
-	bbt_ruler_scale =  bbt_show_many;
-
-	const Beats ceil_upper_beat = std::max (Beats(), tmap->quarters_at_sample (upper)).round_up_to_beat() + Beats (1, 0);
-
-	if (ceil_upper_beat == floor_lower_beat) {
-		return;
-	}
-
-	bbt_bars = count_bars (floor_lower_beat, ceil_upper_beat);
-
-	double ruler_line_granularity = UIConfiguration::instance().get_ruler_granularity ();  //in pixels
-	ruler_line_granularity = _visible_canvas_width / (ruler_line_granularity*5);  //fudge factor '5' probably related to (4+1 beats)/measure, I think
-
-	beats = (ceil_upper_beat - floor_lower_beat).get_beats();
-	double beat_density = ((beats + 1) * ((double) (upper - lower) / (double) (1 + beat_after_upper_pos - beat_before_lower_pos))) / (float)ruler_line_granularity;
-
-	/* Only show the bar helper if there aren't many bars on the screen */
-	if ((bbt_bars < 2) || (beats < 5)) {
-		bbt_bar_helper_on = true;
-	}
-
-	if (beat_density > 2048) {
-		bbt_ruler_scale = bbt_show_many;
-	} else if (beat_density > 1024) {
-		bbt_ruler_scale = bbt_show_64;
-	} else if (beat_density > 256) {
-		bbt_ruler_scale = bbt_show_16;
-	} else if (beat_density > 64) {
-		bbt_ruler_scale = bbt_show_4;
-	} else if (beat_density > 16) {
-		bbt_ruler_scale = bbt_show_1;
-	} else if (beat_density > 4) {
-		bbt_ruler_scale =  bbt_show_quarters;
-	} else  if (beat_density > 2) {
-		bbt_ruler_scale =  bbt_show_eighths;
-	} else  if (beat_density > 1) {
-		bbt_ruler_scale =  bbt_show_sixteenths;
-	} else  if (beat_density > 0.5) {
-		bbt_ruler_scale =  bbt_show_thirtyseconds;
-	} else  if (beat_density > 0.25) {
-		bbt_ruler_scale =  bbt_show_sixtyfourths;
-	} else {
-		bbt_ruler_scale =  bbt_show_onetwentyeighths;
-	}
-
-	/* Now that we know how fine a grid (Ruler) is allowable on this screen, limit it to the coarseness selected by the user */
-	/* note: GridType and RulerScale are not the same enums, so it's not a simple mathematical operation */
-	int suggested_scale = (int) bbt_ruler_scale;
-	int divs = get_grid_music_divisions(_grid_type, 0);
-	if (_grid_type == GridTypeBar) {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_1);
-	} else if (_grid_type == GridTypeBeat) {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_quarters);
-	}  else if ( divs < 4 ) {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_eighths);
-	}  else if ( divs < 8 ) {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_sixteenths);
-	} else if ( divs < 16 ) {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_thirtyseconds);
-	} else if ( divs < 32 ) {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_sixtyfourths);
-	} else {
-		suggested_scale = std::min(suggested_scale, (int) bbt_show_onetwentyeighths);
-	}
-
-	bbt_ruler_scale = (Editor::BBTRulerScale) suggested_scale;
-}
-
 static void
 edit_last_mark_label (std::vector<ArdourCanvas::Ruler::Mark>& marks, const std::string& newlabel)
 {
@@ -1182,7 +1093,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 
 #if 0 // DEBUG GRID
 	for (auto const& g : grid) {
-		std::cout << "Grid " << g.time() <<  " Beats: " << g.beats() << " BBT: " << g.bbt() << " sample: " << g.sample(_session->nominal_sample_rate ()) << "\n";
+		std::cout << "Grid " << g.time() <<  " Beats: " << g.beats() << " BBT: " << g.bbt() << " sample: " << g.sample_is_dangerous (_session->nominal_sample_rate ()) << "\n";
 	}
 #endif
 
@@ -1190,78 +1101,6 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 		return;
 	}
 
-	/* we can accent certain lines depending on the user's Grid choice */
-	/* for example, even in a 4/4 meter we can draw a grid with triplet-feel */
-	/* and in this case you will want the accents on '3s' not '2s' */
-	uint32_t bbt_divisor = 2;
-
-	switch (_grid_type) {
-	case GridTypeBeatDiv3:
-		bbt_divisor = 3;
-		break;
-	case GridTypeBeatDiv5:
-		bbt_divisor = 5;
-		break;
-	case GridTypeBeatDiv6:
-		bbt_divisor = 3;
-		break;
-	case GridTypeBeatDiv7:
-		bbt_divisor = 7;
-		break;
-	case GridTypeBeatDiv10:
-		bbt_divisor = 5;
-		break;
-	case GridTypeBeatDiv12:
-		bbt_divisor = 3;
-		break;
-	case GridTypeBeatDiv14:
-		bbt_divisor = 7;
-		break;
-	case GridTypeBeatDiv16:
-		break;
-	case GridTypeBeatDiv20:
-		bbt_divisor = 5;
-		break;
-	case GridTypeBeatDiv24:
-		bbt_divisor = 6;
-		break;
-	case GridTypeBeatDiv28:
-		bbt_divisor = 7;
-		break;
-	case GridTypeBeatDiv32:
-		break;
-	default:
-		bbt_divisor = 2;
-		break;
-	}
-
-	uint32_t bbt_beat_subdivision = 1;
-	switch (bbt_ruler_scale) {
-	case bbt_show_quarters:
-		bbt_beat_subdivision = 1;
-		break;
-	case bbt_show_eighths:
-		bbt_beat_subdivision = 1;
-		break;
-	case bbt_show_sixteenths:
-		bbt_beat_subdivision = 2;
-		break;
-	case bbt_show_thirtyseconds:
-		bbt_beat_subdivision = 4;
-		break;
-	case bbt_show_sixtyfourths:
-		bbt_beat_subdivision = 8;
-		break;
-	case bbt_show_onetwentyeighths:
-		bbt_beat_subdivision = 16;
-		break;
-	default:
-		bbt_beat_subdivision = 1;
-		break;
-	}
-
-	bbt_beat_subdivision *= bbt_divisor;
-
 	switch (bbt_ruler_scale) {
 
 	case bbt_show_many:
@@ -1289,7 +1128,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 							}
 						}
 						mark.label = buf;
-						mark.position = (*i).sample (sr);
+						mark.position = (*i).sample_is_dangerous (sr);
 						marks.push_back (mark);
 					}
 				}
@@ -1313,7 +1152,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 					}
 				}
 				mark.label = buf;
-				mark.position = (*i).sample(sr);
+				mark.position = (*i).sample_is_dangerous (sr);
 				marks.push_back (mark);
 			  }
 			}
@@ -1333,7 +1172,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 						mark.style = ArdourCanvas::Ruler::Mark::Minor;
 					}
 					mark.label = buf;
-					mark.position = (*i).sample (sr);
+					mark.position = (*i).sample_is_dangerous (sr);
 					marks.push_back (mark);
 				}
 			}
@@ -1347,7 +1186,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 				snprintf (buf, sizeof(buf), "%" PRIu32, bbt.bars);
 				mark.style = ArdourCanvas::Ruler::Mark::Major;
 				mark.label = buf;
-				mark.position = (*i).sample (sr);
+				mark.position = (*i).sample_is_dangerous (sr);
 				marks.push_back (mark);
 			}
 		}
@@ -1364,7 +1203,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 
 			BBT_Time bbt ((*i).bbt());
 
-			if ((*i).sample (sr) < lower && (bbt_bar_helper_on)) {
+			if ((*i).sample_is_dangerous (sr) < lower && (bbt_bar_helper_on)) {
 				snprintf (buf, sizeof(buf), "<%" PRIu32 "|%" PRIu32, bbt.bars, bbt.beats);
 				edit_last_mark_label (marks, buf);
 			} else {
@@ -1380,7 +1219,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 					buf[0] = '\0';
 				}
 				mark.label = buf;
-				mark.position = (*i).sample (sr);
+				mark.position = (*i).sample_is_dangerous (sr);
 				marks.push_back (mark);
 			}
 		}
@@ -1403,7 +1242,7 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 
 			BBT_Time bbt ((*i).bbt());
 
-			if ((*i).sample (sr) < lower && (bbt_bar_helper_on)) {
+			if ((*i).sample_is_dangerous (sr) < lower && (bbt_bar_helper_on)) {
 				snprintf (buf, sizeof(buf), "<%" PRIu32 "|%" PRIu32, bbt.bars, bbt.beats);
 				edit_last_mark_label (marks, buf);
 				helper_active = true;
@@ -1420,11 +1259,11 @@ Editor::metric_get_bbt (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64_t l
 					buf[0] = '\0';
 				}
 
-				if (((*i).sample(sr) < bbt_position_of_helper) && helper_active) {
+				if (((*i).sample_is_dangerous (sr) < bbt_position_of_helper) && helper_active) {
 					buf[0] = '\0';
 				}
 				mark.label =  buf;
-				mark.position = (*i).sample (sr);
+				mark.position = (*i).sample_is_dangerous (sr);
 				marks.push_back (mark);
 			}
 		}
@@ -1463,37 +1302,6 @@ Editor::metric_get_samples (std::vector<ArdourCanvas::Ruler::Mark>& marks, int64
 	}
 }
 
-static void
-sample_to_clock_parts (samplepos_t sample,
-                       samplepos_t sample_rate,
-                       long*       hrs_p,
-                       long*       mins_p,
-                       long*       secs_p,
-                       long*       millisecs_p)
-{
-	samplepos_t left;
-	long hrs;
-	long mins;
-	long secs;
-	long millisecs;
-
-	left = sample;
-	hrs = left / (sample_rate * 60 * 60 * 1000);
-	left -= hrs * sample_rate * 60 * 60 * 1000;
-	mins = left / (sample_rate * 60 * 1000);
-	left -= mins * sample_rate * 60 * 1000;
-	secs = left / (sample_rate * 1000);
-	left -= secs * sample_rate * 1000;
-	millisecs = left / sample_rate;
-
-	*millisecs_p = millisecs;
-	*secs_p = secs;
-	*mins_p = mins;
-	*hrs_p = hrs;
-
-	return;
-}
-
 void
 Editor::set_minsec_ruler_scale (samplepos_t lower, samplepos_t upper)
 {
@@ -1521,22 +1329,22 @@ Editor::set_minsec_ruler_scale (samplepos_t lower, samplepos_t upper)
 		minsec_nmarks = 2 + (range / minsec_mark_interval);
 	} else if (range <= (fr / 2)) { /* 0-0.5 second */
 		minsec_mark_interval = fr / 100;  /* show 1/100 seconds */
-		minsec_ruler_scale = minsec_show_msecs;
+		minsec_ruler_scale = minsec_show_csecs;
 		minsec_mark_modulo = 100;
 		minsec_nmarks = 2 + (range / minsec_mark_interval);
 	} else if (range <= fr) { /* 0-1 second */
 		minsec_mark_interval = fr / 10;  /* show 1/10 seconds */
-		minsec_ruler_scale = minsec_show_msecs;
+		minsec_ruler_scale = minsec_show_dsecs;
 		minsec_mark_modulo = 200;
 		minsec_nmarks = 2 + (range / minsec_mark_interval);
 	} else if (range <= 2 * fr) { /* 1-2 seconds */
 		minsec_mark_interval = fr / 10; /* show 1/10 seconds */
-		minsec_ruler_scale = minsec_show_msecs;
+		minsec_ruler_scale = minsec_show_dsecs;
 		minsec_mark_modulo = 500;
 		minsec_nmarks = 2 + (range / minsec_mark_interval);
 	} else if (range <= 8 * fr) { /* 2-5 seconds */
 		minsec_mark_interval =  fr / 5; /* show 2 seconds */
-		minsec_ruler_scale = minsec_show_msecs;
+		minsec_ruler_scale = minsec_show_dsecs;
 		minsec_mark_modulo = 1000;
 		minsec_nmarks = 2 + (range / minsec_mark_interval);
 	} else if (range <= 16 * fr) { /* 8-16 seconds */
