@@ -16,9 +16,9 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <gtkmm/frame.h>
-#include <gtkmm/label.h>
-#include <gtkmm/scrolledwindow.h>
+#include <ytkmm/frame.h>
+#include <ytkmm/label.h>
+#include <ytkmm/scrolledwindow.h>
 
 #include "ardour/audioengine.h"
 #include "ardour/io_plug.h"
@@ -31,6 +31,7 @@
 #include "gtkmm2ext/menu_elems.h"
 #include "gtkmm2ext/utils.h"
 
+#include "widgets/prompter.h"
 #include "widgets/tooltips.h"
 
 #include "context_menu_helper.h"
@@ -93,14 +94,15 @@ void
 IOPluginWindow::set_session (Session* s)
 {
 	ArdourWindow::set_session (s);
-	_box_pre.set_session (s);
-	_box_post.set_session (s);
 
 	if (!_session) {
 		return;
 	}
+
+	_box_pre.set_session (s);
+	_box_post.set_session (s);
 	refill ();
-	_session->IOPluginsChanged.connect (_session_connections, invalidator (*this), boost::bind (&IOPluginWindow::refill, this), gui_context ());
+	_session->IOPluginsChanged.connect (_session_connections, invalidator (*this), std::bind (&IOPluginWindow::refill, this), gui_context ());
 }
 
 void
@@ -143,7 +145,9 @@ IOPluginWindow::PluginBox::PluginBox (bool is_pre)
 {
 	add_events (Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK);
 	signal_button_press_event ().connect (sigc::mem_fun (*this, &IOPluginWindow::PluginBox::button_press_event));
-	_base.signal_expose_event ().connect (sigc::bind (sigc::ptr_fun (&ArdourWidgets::ArdourIcon::expose_with_text), &_base, ArdourWidgets::ArdourIcon::ShadedPlusSign, _("Right-click or Double-click here\nto add I/O Plugins")));
+	if (UIConfiguration::instance().get_render_plus_hints ()) {
+		_base.signal_expose_event ().connect (sigc::bind (sigc::ptr_fun (&ArdourWidgets::ArdourIcon::expose_with_text), &_base, ArdourWidgets::ArdourIcon::ShadedPlusSign, _("Right-click or Double-click here\nto add I/O Plugins")));
+	}
 
 	std::vector<Gtk::TargetEntry> target_table;
 	target_table.push_back (Gtk::TargetEntry ("x-ardour/plugin.favorite", Gtk::TARGET_SAME_APP)); // from sidebar
@@ -310,7 +314,7 @@ IOPluginWindow::IOPlugUI::IOPlugUI (std::shared_ptr<ARDOUR::IOPlug> iop)
 	}
 
 	_btn_ioplug.signal_button_press_event ().connect (sigc::mem_fun (*this, &IOPluginWindow::IOPlugUI::button_press_event), false);
-	_iop->DropReferences.connect (_going_away_connection, invalidator (*this), boost::bind (&IOPluginWindow::IOPlugUI::self_delete, this), gui_context ());
+	_iop->DropReferences.connect (_going_away_connection, invalidator (*this), std::bind (&IOPluginWindow::IOPlugUI::self_delete, this), gui_context ());
 	show_all ();
 }
 
@@ -328,6 +332,32 @@ IOPluginWindow::IOPlugUI::self_remove ()
 	_iop->session ().unload_io_plugin (_iop); // this calls self_delete()
 }
 
+void
+IOPluginWindow::IOPlugUI::rename ()
+{
+	ArdourWidgets::Prompter name_prompter (true);
+	name_prompter.set_title (_("Rename I/O Plugin"));
+	name_prompter.set_prompt (_("New name:"));
+	name_prompter.set_initial_text (_iop->name());
+	name_prompter.add_button (_("Rename"), Gtk::RESPONSE_ACCEPT);
+	name_prompter.set_response_sensitive (Gtk::RESPONSE_ACCEPT, false);
+	name_prompter.show_all ();
+
+	std::string result;
+	switch (name_prompter.run ()) {
+		case Gtk::RESPONSE_ACCEPT:
+			name_prompter.get_result (result);
+			name_prompter.hide ();
+			if (result.length ()) {
+				_iop->set_name (result);
+				_btn_ioplug.set_text (_iop->name ());
+			}
+			break;
+		default:
+			break;
+	}
+}
+
 void
 IOPluginWindow::IOPlugUI::edit_plugin (bool custom_ui)
 {
@@ -351,6 +381,8 @@ IOPluginWindow::IOPlugUI::button_press_event (GdkEventButton* ev)
 		Gtk::Menu* m = ARDOUR_UI_UTILS::shared_popup_menu ();
 		MenuList& items = m->items ();
 
+		items.push_back (MenuElem (_("Rename"), sigc::mem_fun (*this, &IOPluginWindow::IOPlugUI::rename)));
+		items.push_back (SeparatorElem());
 		items.push_back (MenuElem (_("Edit..."), sigc::bind (sigc::mem_fun (*this, &IOPluginWindow::IOPlugUI::edit_plugin), true)));
 		items.back().set_sensitive (_iop->plugin ()->has_editor ());
 		items.push_back (MenuElem (_("Edit with generic controls..."), sigc::bind (sigc::mem_fun (*this, &IOPluginWindow::IOPlugUI::edit_plugin), false)));
@@ -392,11 +424,11 @@ IOPluginWindow::IOButton::IOButton (std::shared_ptr<ARDOUR::IO> io, bool pre)
 	signal_button_press_event ().connect (sigc::mem_fun (*this, &IOButton::button_press), false);
 	signal_button_release_event ().connect (sigc::mem_fun (*this, &IOButton::button_release), false);
 
-	AudioEngine::instance ()->PortConnectedOrDisconnected.connect (_connections, invalidator (*this), boost::bind (&IOButton::port_connected_or_disconnected, this, _1, _3), gui_context ());
-	AudioEngine::instance ()->PortPrettyNameChanged.connect (_connections, invalidator (*this), boost::bind (&IOButton::port_pretty_name_changed, this, _1), gui_context ());
+	AudioEngine::instance ()->PortConnectedOrDisconnected.connect (_connections, invalidator (*this), std::bind (&IOButton::port_connected_or_disconnected, this, _1, _3), gui_context ());
+	AudioEngine::instance ()->PortPrettyNameChanged.connect (_connections, invalidator (*this), std::bind (&IOButton::port_pretty_name_changed, this, _1), gui_context ());
 
-	_io->changed.connect (_connections, invalidator (*this), boost::bind (&IOButton::update, this), gui_context ());
-	_io->session ().BundleAddedOrRemoved.connect (_connections, invalidator (*this), boost::bind (&IOButton::update, this), gui_context ());
+	_io->changed.connect (_connections, invalidator (*this), std::bind (&IOButton::update, this), gui_context ());
+	_io->session ().BundleAddedOrRemoved.connect (_connections, invalidator (*this), std::bind (&IOButton::update, this), gui_context ());
 }
 
 IOPluginWindow::IOButton::~IOButton ()
@@ -444,7 +476,7 @@ IOPluginWindow::IOButton::update ()
 	set_label (*this, _io->session (), bundle, _io);
 
 	if (bundle) {
-		bundle->Changed.connect (_bundle_connections, invalidator (*this), boost::bind (&IOButton::update, this), gui_context ());
+		bundle->Changed.connect (_bundle_connections, invalidator (*this), std::bind (&IOButton::update, this), gui_context ());
 	}
 }
 
