@@ -22,22 +22,21 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef __ardour_mixer_ui_h__
-#define __ardour_mixer_ui_h__
+#pragma once
 
 #include <list>
 
-#include <gtkmm/box.h>
-#include <gtkmm/scrolledwindow.h>
-#include <gtkmm/eventbox.h>
-#include <gtkmm/label.h>
-#include <gtkmm/comboboxtext.h>
-#include <gtkmm/button.h>
-#include <gtkmm/frame.h>
-#include <gtkmm/menu.h>
-#include <gtkmm/treeview.h>
-#include <gtkmm/treestore.h>
-#include <gtkmm/liststore.h>
+#include <ytkmm/box.h>
+#include <ytkmm/scrolledwindow.h>
+#include <ytkmm/eventbox.h>
+#include <ytkmm/label.h>
+#include <ytkmm/comboboxtext.h>
+#include <ytkmm/button.h>
+#include <ytkmm/frame.h>
+#include <ytkmm/menu.h>
+#include <ytkmm/treeview.h>
+#include <ytkmm/treestore.h>
+#include <ytkmm/liststore.h>
 
 #include "pbd/stateful.h"
 #include "pbd/signals.h"
@@ -52,10 +51,13 @@
 #include "gtkmm2ext/dndtreeview.h"
 #include "gtkmm2ext/treeutils.h"
 
+#include "widgets/ardour_dropdown.h"
+#include "widgets/frame.h"
+#include "widgets/metabutton.h"
 #include "widgets/pane.h"
 #include "widgets/tabbable.h"
-#include "widgets/ardour_dropdown.h"
 
+#include "application_bar.h"
 #include "axis_provider.h"
 #include "enums.h"
 #include "monitor_section.h"
@@ -144,11 +146,14 @@ public:
 
 	void register_actions ();
 
+	void focus_on_clock();
+
 	void load_bindings ();
 	Gtkmm2ext::Bindings*  bindings;
 
 	void toggle_mixer_list ();
-	void showhide_mixer_list (bool yn);
+	void toggle_mixer_strip ();
+	void toggle_mixer_props ();
 
 	void toggle_surround_master ();
 
@@ -170,14 +175,32 @@ public:
 
 	void toggle_monitor_action (ARDOUR::MonitorChoice monitor_choice, bool group_override = false, bool all = false);
 
+	class RedisplaySuspender {
+		public:
+			RedisplaySuspender() {
+				Mixer_UI::instance()->no_track_list_redisplay = true;
+				Mixer_UI::instance()->ignore_track_reorder = true;
+				Mixer_UI::instance()->_in_group_rebuild_or_clear = true;
+			}
+			~RedisplaySuspender () {
+				Mixer_UI::instance()->no_track_list_redisplay = false;
+				Mixer_UI::instance()->ignore_track_reorder = false;
+				Mixer_UI::instance()->redisplay_track_list ();
+				Mixer_UI::instance()->redisplay_track_list ();
+				Mixer_UI::instance()->route_groups_changed ();
+			}
+	};
+
 protected:
+	friend class RedisplaySuspender;
+
 	void set_axis_targets_for_operation ();
 	PBD::ControllableSet selected_gaincontrols ();
 
 private:
 	Mixer_UI ();
 	static Mixer_UI*     _instance;
-	Gtk::VBox            _content;
+
 	Gtk::HBox             global_hpacker;
 	Gtk::VBox             global_vpacker;
 	Gtk::ScrolledWindow   scroller;
@@ -190,16 +213,10 @@ private:
 	Gtk::ScrolledWindow   group_display_scroller;
 	Gtk::ScrolledWindow   favorite_plugins_scroller;
 	Gtk::VBox             group_display_vbox;
-	Gtk::Frame            track_display_frame;
-	Gtk::Frame            group_display_frame;
-	Gtk::Frame            favorite_plugins_frame;
 	Gtk::VBox             favorite_plugins_vbox;
 	Gtk::HBox             favorite_plugins_search_hbox;
-	ArdourWidgets::ArdourDropdown favorite_plugins_mode_combo;
 	Gtk::Entry            plugin_search_entry;
 	ArdourWidgets::ArdourButton plugin_search_clear_button;
-	ArdourWidgets::VPane  rhs_pane1;
-	ArdourWidgets::VPane  rhs_pane2;
 	ArdourWidgets::HPane  inner_pane;
 	Gtk::VBox             strip_group_box;
 	Gtk::HBox             strip_packer;
@@ -210,10 +227,13 @@ private:
 	Gtk::Label            vca_label;
 	Gtk::EventBox         vca_scroller_base;
 	Gtk::HBox             out_packer;
-	ArdourWidgets::HPane  list_hpane;
+
+	Gtk::Notebook             _sidebar_notebook;
+	ArdourWidgets::MetaButton _sidebar_pager1;
+	ArdourWidgets::MetaButton _sidebar_pager2;
 
 	Gtk::EventBox         _mixer_scene_spacer;
-	Gtk::Frame            _mixer_scene_frame;
+	ArdourWidgets::Frame  _mixer_scene_frame;
 	Gtk::Table            _mixer_scene_table;
 	Gtk::VBox             _mixer_scene_vbox;
 
@@ -248,6 +268,8 @@ private:
 
 	void update_scene_buttons ();
 
+	void add_sidebar_page (std::string const&, std::string const&, Gtk::Widget&);
+
 	void add_routes (ARDOUR::RouteList&);
 	void remove_strip (MixerStrip *);
 	void remove_foldback (FoldbackStrip *);
@@ -325,7 +347,7 @@ private:
 	void remove_selected_route_group ();
 	void activate_all_route_groups ();
 	void disable_all_route_groups ();
-	void add_route_group (ARDOUR::RouteGroup *);
+	void add_route_group (std::shared_ptr<ARDOUR::RouteGroup>);
 	void route_groups_changed ();
 	void route_group_name_edit (const std::string&, const std::string&);
 	void route_group_row_change (const Gtk::TreeModel::Path& path,const Gtk::TreeModel::iterator& iter);
@@ -335,6 +357,8 @@ private:
 	void track_column_click (gint);
 	void build_track_menu ();
 
+	ApplicationBar _application_bar;
+
 	MonitorSection   _monitor_section;
 	PluginSelector* _plugin_selector;
 	SurroundStrip*  _surround_strip;
@@ -342,7 +366,7 @@ private:
 	bool _show_foldback_strip;
 
 	void stripable_property_changed (const PBD::PropertyChange& what_changed, std::weak_ptr<ARDOUR::Stripable> ws);
-	void route_group_property_changed (ARDOUR::RouteGroup *, const PBD::PropertyChange &);
+	void route_group_property_changed (std::weak_ptr<ARDOUR::RouteGroup>, const PBD::PropertyChange &);
 
 	/* various treeviews */
 
@@ -367,7 +391,7 @@ private:
 		}
 		Gtk::TreeModelColumn<bool>            visible;
 		Gtk::TreeModelColumn<std::string>         text;
-		Gtk::TreeModelColumn<ARDOUR::RouteGroup*> group;
+		Gtk::TreeModelColumn<std::shared_ptr<ARDOUR::RouteGroup>> group;
 	};
 
 	struct PluginsDisplayModelColumns : public Gtk::TreeModel::ColumnRecord {
@@ -414,7 +438,7 @@ private:
 	bool ignore_plugin_reorder;
 
 	void parameter_changed (std::string const &);
-	void set_route_group_activation (ARDOUR::RouteGroup *, bool);
+	void set_route_group_activation (std::shared_ptr<ARDOUR::RouteGroup>, bool);
 
 	void setup_track_display ();
 	void new_track_or_bus ();
@@ -442,11 +466,12 @@ private:
 	enum PluginListMode {
 		PLM_Favorite,
 		PLM_Recent,
-		PLM_TopHits
+		PLM_TopHits,
+		PLM_SearchAll
 	};
 	enum PluginListMode plugin_list_mode;
 	void set_plugin_list_mode (PluginListMode plm);
-	std::vector<std::string> plugin_list_mode_strings;
+	void update_sidebar_pagers (guint);
 
 	void refiller (ARDOUR::PluginInfoList& result, const ARDOUR::PluginInfoList& plugs);
 	void refill_favorite_plugins ();
@@ -493,5 +518,3 @@ private:
 	void toggle_processors ();
 	void ab_plugins ();
 };
-
-#endif /* __ardour_mixer_ui_h__ */
