@@ -21,7 +21,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <gdkmm/pixbuf.h>
+#include <ydkmm/pixbuf.h>
 
 #include "pbd/compose.h"
 #include "pbd/error.h"
@@ -30,8 +30,8 @@
 #include "gtkmm2ext/actions.h"
 #include "gtkmm2ext/utils.h"
 
-#include <gtkmm/menu.h>
-#include <gtkmm/menuitem.h>
+#include <ytkmm/menu.h>
+#include <ytkmm/menuitem.h>
 
 #include "widgets/tearoff.h"
 #include "widgets/tooltips.h"
@@ -107,11 +107,11 @@ MonitorSection::MonitorSection ()
 
 	load_bindings ();
 	register_actions ();
-	set_data ("ardour-bindings", bindings);
+	set_widget_bindings (*this, *bindings, ARDOUR_BINDING_KEY);
 
 	channel_size_group = SizeGroup::create (SIZE_GROUP_HORIZONTAL);
 
-	insert_box = new ProcessorBox (0, boost::bind (&MonitorSection::plugin_selector, this), _rr_selection, 0);
+	insert_box = new ProcessorBox (0, std::bind (&MonitorSection::plugin_selector, this), _rr_selection, 0);
 	insert_box->set_no_show_all ();
 	insert_box->show ();
 	// TODO allow keyboard shortcuts in ProcessorBox
@@ -490,11 +490,12 @@ MonitorSection::MonitorSection ()
 	_tearoff->tearoff_window().set_title (X_("Monitor"));
 	_tearoff->tearoff_window().signal_key_press_event().connect (sigc::bind (sigc::ptr_fun (relay_key_press), (Gtk::Window*) &_tearoff->tearoff_window()), false);
 
+	unassign_controllables ();
 	update_processor_box ();
 	_ui_initialized = true;
 
 	/* catch changes that affect us */
-	Config->ParameterChanged.connect (config_connection, invalidator (*this), boost::bind (&MonitorSection::parameter_changed, this, _1), gui_context());
+	Config->ParameterChanged.connect (config_connection, invalidator (*this), std::bind (&MonitorSection::parameter_changed, this, _1), gui_context());
 }
 
 MonitorSection::~MonitorSection ()
@@ -584,12 +585,13 @@ void
 MonitorSection::set_session (Session* s)
 {
 	RouteUI::set_session (s);
-	insert_box->set_session (_session);
 
 	Glib::RefPtr<ActionGroup> global_monitor_actions = ActionManager::get_action_group (X_("Monitor Section"));
 
 	if (_session) {
 
+		insert_box->set_session (_session);
+
 		/* These are not actually dependent on the Session, but they
 		 * need to be set after construction, not during, and
 		 * this is as good a place as any.
@@ -606,9 +608,9 @@ MonitorSection::set_session (Session* s)
 			_monitor = _route->monitor_control ();
 			assign_controllables ();
 			insert_box->set_route (_route);
-			_route->processors_changed.connect (route_connections, invalidator (*this), boost::bind (&MonitorSection::processors_changed, this, _1), gui_context());
-			_route->output()->PortCountChanged.connect (route_connections, invalidator (*this), boost::bind (&MonitorSection::populate_buttons, this), gui_context());
-			_route->DropReferences.connect (route_connections, invalidator (*this), boost::bind (&MonitorSection::drop_route, this), gui_context());
+			_route->processors_changed.connect (route_connections, invalidator (*this), std::bind (&MonitorSection::processors_changed, this, _1), gui_context());
+			_route->output()->PortCountChanged.connect (route_connections, invalidator (*this), std::bind (&MonitorSection::populate_buttons, this), gui_context());
+			_route->DropReferences.connect (route_connections, invalidator (*this), std::bind (&MonitorSection::drop_route, this), gui_context());
 
 			if (_ui_initialized) {
 				update_processor_box ();
@@ -709,6 +711,7 @@ MonitorSection::populate_buttons ()
 	channel_table->resize (nchans, 5);
 
 	const uint32_t row_offset = 0;
+	int channel_errors = 0;
 
 	for (uint32_t i = 0; i < nchans; ++i) {
 
@@ -739,31 +742,49 @@ MonitorSection::populate_buttons ()
 		channel_table->attach (cbs->solo, 3, 4, i+row_offset, i+row_offset+1, EXPAND|FILL);
 		channel_table->attach (cbs->invert, 4, 5, i+row_offset, i+row_offset+1, EXPAND|FILL);
 
-		snprintf (buf, sizeof (buf), "monitor-cut-%u", i);
-		act = ActionManager::get_action (X_("Monitor"), buf);
-		if (act) {
-			cbs->cut.set_related_action (act);
-		}
+		try {
+			snprintf (buf, sizeof (buf), "monitor-cut-%u", i);
+			act = ActionManager::get_action (X_("Monitor"), buf, false);
+			/* could have failed if channel count is too high, so try to
+			   register it, and then try again. If it worked, it will work
+			   for all per-channel actions; if not, it will throw an
+			   exception
+			*/
+			if (!act) {
+				register_channel_actions (i);
+				act = ActionManager::get_action (X_("Monitor"), buf);
+			}
 
-		snprintf (buf, sizeof (buf), "monitor-dim-%u", i);
-		act = ActionManager::get_action (X_("Monitor"), buf);
-		if (act) {
-			cbs->dim.set_related_action (act);
-		}
+			if (act) {
+				cbs->cut.set_related_action (act);
+			}
 
-		snprintf (buf, sizeof (buf), "monitor-solo-%u", i);
-		act = ActionManager::get_action (X_("Monitor"), buf);
-		if (act) {
-			cbs->solo.set_related_action (act);
-		}
+			snprintf (buf, sizeof (buf), "monitor-dim-%u", i);
+			act = ActionManager::get_action (X_("Monitor"), buf);
+			if (act) {
+				cbs->dim.set_related_action (act);
+			}
 
-		snprintf (buf, sizeof (buf), "monitor-invert-%u", i);
-		act = ActionManager::get_action (X_("Monitor"), buf);
-		if (act) {
-			cbs->invert.set_related_action (act);
+			snprintf (buf, sizeof (buf), "monitor-solo-%u", i);
+			act = ActionManager::get_action (X_("Monitor"), buf);
+			if (act) {
+				cbs->solo.set_related_action (act);
+			}
+
+			snprintf (buf, sizeof (buf), "monitor-invert-%u", i);
+			act = ActionManager::get_action (X_("Monitor"), buf);
+			if (act) {
+				cbs->invert.set_related_action (act);
+			}
+		} catch (...) {
+			channel_errors++;
 		}
 	}
 
+	if (channel_errors) {
+		error << _("Some per channel monitor buttons will not work due to missing actions") << std::endl;
+	}
+
 	channel_table->show_all ();
 
 	if (channel_table_scroller.get_parent()) {
@@ -943,27 +964,7 @@ MonitorSection::register_actions ()
 
 
 	for (uint32_t chn = 0; chn < 16; ++chn) {
-
-		action_name = string_compose (X_("monitor-cut-%1"), chn);
-		action_descr = string_compose (_("Cut monitor channel %1"), chn);
-		ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
-		                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::cut_channel), chn));
-
-		action_name = string_compose (X_("monitor-dim-%1"), chn);
-		action_descr = string_compose (_("Dim monitor channel %1"), chn);
-		ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
-		                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::dim_channel), chn));
-
-		action_name = string_compose (X_("monitor-solo-%1"), chn);
-		action_descr = string_compose (_("Solo monitor channel %1"), chn);
-		ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
-		                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::solo_channel), chn));
-
-		action_name = string_compose (X_("monitor-invert-%1"), chn);
-		action_descr = string_compose (_("Invert monitor channel %1"), chn);
-		ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
-		                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::invert_channel), chn));
-
+		register_channel_actions (chn);
 	}
 
 	solo_actions = ActionManager::create_action_group (bindings, X_("Solo"));
@@ -982,6 +983,34 @@ MonitorSection::register_actions ()
 	                                       sigc::mem_fun (*this, &MonitorSection::toggle_mute_overrides_solo));
 }
 
+void
+MonitorSection::register_channel_actions (uint32_t chn)
+{
+	string action_name;
+	string action_descr;
+	Glib::RefPtr<Action> act;
+
+	action_name = string_compose (X_("monitor-cut-%1"), chn);
+	action_descr = string_compose (_("Cut monitor channel %1"), chn);
+	ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
+	                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::cut_channel), chn));
+
+	action_name = string_compose (X_("monitor-dim-%1"), chn);
+	action_descr = string_compose (_("Dim monitor channel %1"), chn);
+	ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
+	                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::dim_channel), chn));
+
+	action_name = string_compose (X_("monitor-solo-%1"), chn);
+	action_descr = string_compose (_("Solo monitor channel %1"), chn);
+	ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
+	                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::solo_channel), chn));
+
+	action_name = string_compose (X_("monitor-invert-%1"), chn);
+	action_descr = string_compose (_("Invert monitor channel %1"), chn);
+	ActionManager::register_toggle_action (monitor_actions, action_name.c_str(), action_descr.c_str(),
+	                                       sigc::bind (sigc::mem_fun (*this, &MonitorSection::invert_channel), chn));
+}
+
 void
 MonitorSection::solo_use_in_place ()
 {
@@ -1216,6 +1245,23 @@ MonitorSection::unassign_controllables ()
 	dim_display->set_controllable (none);
 	solo_boost_control->set_controllable (none);
 	solo_boost_display->set_controllable (none);
+
+	Glib::RefPtr<ToggleAction> tact;
+	tact = ActionManager::get_toggle_action (X_("Monitor Section"), X_("monitor-dim-all"));
+	if (tact) {
+		tact->set_active (false);
+		tact->set_sensitive (false);
+	}
+	tact = ActionManager::get_toggle_action (X_("Monitor Section"), X_("monitor-mono"));
+	if (tact) {
+		tact->set_active (false);
+		tact->set_sensitive (false);
+	}
+	tact = ActionManager::get_toggle_action (X_("Monitor Section"), X_("monitor-cut-all"));
+	if (tact) {
+		tact->set_active (false);
+		tact->set_sensitive (false);
+	}
 }
 
 void
@@ -1240,6 +1286,21 @@ MonitorSection::assign_controllables ()
 	dim_display->set_controllable (_monitor->dim_level_control ());
 	solo_boost_control->set_controllable (_monitor->solo_boost_control ());
 	solo_boost_display->set_controllable (_monitor->solo_boost_control ());
+
+
+	Glib::RefPtr<Action> act;
+	act = ActionManager::get_action (X_("Monitor Section"), X_("monitor-dim-all"));
+	if (act) {
+		act->set_sensitive (true);
+	}
+	act = ActionManager::get_action (X_("Monitor Section"), X_("monitor-mono"));
+	if (act) {
+		act->set_sensitive (true);
+	}
+	act = ActionManager::get_action (X_("Monitor Section"), X_("monitor-cut-all"));
+	if (act) {
+		act->set_sensitive (true);
+	}
 }
 
 string
