@@ -26,7 +26,7 @@
 
 #include <cstdio> // for snprintf, grrr
 
-#include <gtkmm/stock.h>
+#include <ytkmm/stock.h>
 
 #include "pbd/error.h"
 #include "midi++/parser.h"
@@ -66,8 +66,9 @@ TempoDialog::TempoDialog (TempoMap::SharedPtr const & map, timepos_t const & pos
 {
 	Temporal::BBT_Time when (_map->bbt_at (pos));
 	Tempo const & tempo (_map->tempo_at (pos));
+	Meter const & meter (_map->meter_at (pos));
 
-	init (when, tempo.note_types_per_minute(), tempo.end_note_types_per_minute(), tempo.note_type(), Tempo::Constant, true, BeatTime);
+	init (when, tempo.note_types_per_minute(), tempo.end_note_types_per_minute(), meter.note_value(), Tempo::Constant, true, BeatTime);
 }
 
 TempoDialog::TempoDialog (TempoMap::SharedPtr const & map, TempoPoint& point, const string&)
@@ -86,7 +87,9 @@ TempoDialog::TempoDialog (TempoMap::SharedPtr const & map, TempoPoint& point, co
 	, tap_tempo_button (_("Tap tempo"))
 {
 	Temporal::BBT_Time when (map->bbt_at (point.time()));
-	init (when, _section->note_types_per_minute(), _section->end_note_types_per_minute(), _section->note_type(), _section->type(), map->is_initial (point), Temporal::BeatTime);
+	Meter const & meter (_map->meter_at (point.time()));
+
+	init (when, _section->note_types_per_minute(), _section->end_note_types_per_minute(), meter.note_value(), _section->type(), map->is_initial (point), Temporal::BeatTime);
 }
 
 void
@@ -111,8 +114,8 @@ TempoDialog::init (const Temporal::BBT_Time& when, double bpm, double end_bpm, d
 
 	note_types.insert (make_pair (_("whole"), 1));
 	strings.push_back (_("whole"));
-	note_types.insert (make_pair (_("second"), 2));
-	strings.push_back (_("second"));
+	note_types.insert (make_pair (_("half"), 2));
+	strings.push_back (_("half"));
 	note_types.insert (make_pair (_("third"), 3));
 	strings.push_back (_("third"));
 	note_types.insert (make_pair (_("quarter"), 4));
@@ -184,7 +187,7 @@ TempoDialog::init (const Temporal::BBT_Time& when, double bpm, double end_bpm, d
 
 	int row = 0;
 
-	if (UIConfiguration::instance().get_allow_non_quarter_pulse()) {
+	if (UIConfiguration::instance().get_allow_non_quarter_pulse() || note_type != 4) {
 		table->attach (pulse_selector_label, 0, 1, row, row + 1);
 		table->attach (pulse_selector, 1, 5, row, row + 1);
 
@@ -289,14 +292,14 @@ TempoDialog::init (const Temporal::BBT_Time& when, double bpm, double end_bpm, d
 	_midi_port_combo.set_model (_midi_port_list);
 	_midi_port_combo.pack_start (_midi_port_cols.pretty_name);
 
-	AudioEngine::instance ()->PortRegisteredOrUnregistered.connect (_manager_connection, invalidator (*this), boost::bind (&TempoDialog::ports_changed, this), gui_context ());
+	AudioEngine::instance ()->PortRegisteredOrUnregistered.connect (_manager_connection, invalidator (*this), std::bind (&TempoDialog::ports_changed, this), gui_context ());
 	std::shared_ptr<Port> port = AudioEngine::instance ()->register_input_port (DataType::MIDI, "Tap Tempo", false, PortFlags (IsInput | Hidden | IsTerminal));
 	_midi_tap_port               = std::dynamic_pointer_cast<MidiPort> (port);
 	assert (_midi_tap_port);
 	_midi_tap_parser = std::shared_ptr<MIDI::Parser> (new MIDI::Parser);
-	_midi_tap_parser->any.connect_same_thread (_parser_connection, boost::bind (&TempoDialog::midi_event, this, _2, _3, _4));
+	_midi_tap_parser->any.connect_same_thread (_parser_connection, std::bind (&TempoDialog::midi_event, this, _2, _3, _4));
 	_midi_tap_port->set_trace (_midi_tap_parser);
-	_midi_tap_signal.connect (_xthread_connection, invalidator (*this), boost::bind (&TempoDialog::tap_tempo, this, _1), gui_context ());
+	_midi_tap_signal.connect (_xthread_connection, invalidator (*this), std::bind (&TempoDialog::tap_tempo, this, _1), gui_context ());
 
 	/* init state */
 	tempo_type_change ();
@@ -496,7 +499,7 @@ TempoDialog::get_note_type ()
 
 	if (x == note_types.end()) {
 		error << string_compose(_("incomprehensible pulse note type (%1)"), pulse_selector.get_active_text()) << endmsg;
-		return 0;
+		return 4;
 	}
 
 	return x->second;
