@@ -18,11 +18,11 @@
 
 #include <glib/gstdio.h>
 
-#include <gtkmm/alignment.h>
-#include <gtkmm/filechooserdialog.h>
-#include <gtkmm/menu.h>
-#include <gtkmm/menuitem.h>
-#include <gtkmm/stock.h>
+#include <ytkmm/alignment.h>
+#include <ytkmm/filechooserdialog.h>
+#include <ytkmm/menu.h>
+#include <ytkmm/menuitem.h>
+#include <ytkmm/stock.h>
 
 #include "pbd/compose.h"
 #include "pbd/convert.h"
@@ -40,18 +40,20 @@
 #include "ardour/triggerbox.h"
 
 #include "gtkmm2ext/colors.h"
+#include <gtkmm2ext/utils.h>
 
-#include "slot_properties_box.h"
+#include "widgets/popup.h"
 
+#include "ardour_color_dialog.h"
 #include "ardour_ui.h"
 #include "gui_thread.h"
 #include "keyboard.h"
 #include "public_editor.h"
 #include "region_view.h"
+#include "slot_properties_box.h"
 #include "trigger_jump_dialog.h"
-#include "ui_config.h"
-
 #include "trigger_ui.h"
+#include "ui_config.h"
 
 #include "pbd/i18n.h"
 
@@ -129,46 +131,49 @@ TriggerUI::~TriggerUI()
 void
 TriggerUI::trigger_swap (uint32_t n)
 {
-	if (n != tref.slot) {
+	if (n != tref.slot()) {
 		/* some other slot in the same box got swapped. we don't care */
 		return;
 	}
 	trigger_connections.drop_connections ();
 
-	trigger()->PropertyChanged.connect (trigger_connections, invalidator (*this), boost::bind (&TriggerUI::trigger_changed, this, _1), gui_context ());
-	tref.box->PropertyChanged.connect (trigger_connections, invalidator (*this), boost::bind (&TriggerUI::trigger_changed, this, _1), gui_context ());
+	trigger()->PropertyChanged.connect (trigger_connections, invalidator (*this), std::bind (&TriggerUI::trigger_changed, this, _1), gui_context ());
+	tref.box()->PropertyChanged.connect (trigger_connections, invalidator (*this), std::bind (&TriggerUI::trigger_changed, this, _1), gui_context ());
 
 	trigger_changed (Properties::name);
 }
 
 void
-TriggerUI::choose_color ()
+TriggerUI::choose_color (Gtk::Window* parent)
 {
 	if (!_color_dialog) {
-		_color_dialog = new Gtk::ColorSelectionDialog;
+		_color_dialog = new ArdourColorDialog;
+		tref.box()->DropReferences.connect (trigger_connections, invalidator (*this), [this]() { delete _color_dialog; _color_dialog = nullptr; }, gui_context());
 	}
 
-	_color_dialog->get_color_selection()->set_has_opacity_control (false);
-	_color_dialog->get_color_selection()->set_has_palette (true);
-	_color_dialog->get_ok_button()->signal_clicked().connect (sigc::bind (sigc::mem_fun (_color_dialog, &Gtk::Dialog::response), Gtk::RESPONSE_ACCEPT));
-	_color_dialog->get_cancel_button()->signal_clicked().connect (sigc::bind (sigc::mem_fun (_color_dialog, &Gtk::Dialog::response), Gtk::RESPONSE_CANCEL));
-
-	Gdk::Color c = Gtkmm2ext::gdk_color_from_rgba(trigger()->color());
-
-	_color_dialog->get_color_selection()->set_previous_color (c);
+	_color_dialog->set_title (_("Set Trigger Color"));
+	Gdk::Color c (Gtkmm2ext::gdk_color_from_rgba (trigger()->color()));
 	_color_dialog->get_color_selection()->set_current_color (c);
+	_color_dialog->get_color_selection()->set_previous_color (c);
+	if (parent) {
+		_color_dialog->set_transient_for (*parent);
+	}
 
-	switch (_color_dialog->run()) {
-		case Gtk::RESPONSE_ACCEPT: {
-			c = _color_dialog->get_color_selection()->get_current_color();
-			color_t ct = Gtkmm2ext::gdk_color_to_rgba(c);
-			trigger()->set_color(ct);
-		} break;
+	_color_dialog->signal_response().connect ([this](int response) {
+		if (!trigger()) {
+			return;
+		}
+		switch (response) {
+		case Gtk::RESPONSE_OK:
+			trigger()->set_color (Gtkmm2ext::gdk_color_to_rgba (_color_dialog->get_color_selection()->get_current_color()));
+			break;
 		default:
 			break;
-	}
+		}
+		_color_dialog->hide ();
+	});
 
-	_color_dialog->hide ();
+	_color_dialog->present ();
 }
 
 void
@@ -404,14 +409,22 @@ TriggerUI::context_menu ()
 
 	items.push_back (MenuElem (_("Load..."), sigc::bind(sigc::mem_fun (*this, (&TriggerUI::choose_sample)), true)));
 	items.push_back (SeparatorElem());
-	items.push_back (MenuElem (_("Color..."), sigc::mem_fun (*this, &TriggerUI::choose_color)));
+	items.push_back (MenuElem (_("Color..."), sigc::bind (sigc::mem_fun (*this, &TriggerUI::choose_color), (Gtk::Window*)NULL)));
 	items.push_back (MenuElem (_("Clear"), sigc::mem_fun (*this, &TriggerUI::clear_trigger)));
 	items.push_back (SeparatorElem());
 	items.push_back (MenuElem (_("MIDI Learn"), sigc::mem_fun (*this, &TriggerUI::trigger_midi_learn)));
 	items.push_back (MenuElem (_("MIDI un-Learn"), sigc::mem_fun (*this, &TriggerUI::trigger_midi_unlearn)));
 
+	_context_menu->popup (1, gtk_get_current_event_time ());
+}
 
-	_context_menu->popup (3, gtk_get_current_event_time ());
+void
+TriggerUI::trigger_learning_finished (ArdourWidgets::PopUp* prompter)
+{
+	learning_connection.disconnect ();
+	if (prompter) {
+		prompter->touch ();
+	}
 }
 
 void
@@ -421,13 +434,23 @@ TriggerUI::trigger_midi_learn ()
 		return;
 	}
 
-	tref.box->begin_midi_learn (trigger()->index());
+	ArdourWidgets::PopUp* prompter = new ArdourWidgets::PopUp (Gtk::WIN_POS_MOUSE, 30000, true);
+	prompter->set_text (_("operate controller now"));
+	prompter->touch (); // shows popup
+	tref.box()->TriggerMIDILearned.connect (learning_connection, invalidator (*this), std::bind (&TriggerUI::trigger_learning_finished, this, prompter), gui_context());
+	tref.box()->begin_midi_learn (trigger()->index());
+}
+
+void
+TriggerUI::trigger_stop_midi_learn ()
+{
+	tref.box()->stop_midi_learn ();
 }
 
 void
 TriggerUI::trigger_midi_unlearn ()
 {
-	tref.box->midi_unlearn (trigger()->index());
+	tref.box()->midi_unlearn (trigger()->index());
 }
 
 void
@@ -546,7 +569,7 @@ TriggerUI::launch_context_menu ()
 		dynamic_cast<Gtk::CheckMenuItem*> (&items.back ())->set_active (true);
 	}
 
-	_launch_context_menu->popup (3, gtk_get_current_event_time ());
+	_launch_context_menu->popup (1, gtk_get_current_event_time ());
 }
 
 void
@@ -627,14 +650,6 @@ TriggerUI::clear_trigger ()
 void
 TriggerUI::edit_trigger ()
 {
-	SlotPropertyWindow* tw      = static_cast<SlotPropertyWindow*> (trigger()->ui ());
-
-	if (!tw) {
-		tw = new SlotPropertyWindow (TriggerReference (trigger()->box(), trigger()->index()));
-		trigger()->set_ui (tw);
-	}
-
-	tw->present ();
 }
 
 void
@@ -797,7 +812,6 @@ TriggerUI::trigger_changed (PropertyChange const& what)
 	on_trigger_changed(what);
 }
 
-
 void
 TriggerUI::set_trigger (ARDOUR::TriggerReference tr)
 {
@@ -808,10 +822,21 @@ TriggerUI::set_trigger (ARDOUR::TriggerReference tr)
 
 	trigger_changed (TriggerBox::all_trigger_props());
 
-	trigger()->PropertyChanged.connect (trigger_connections, invalidator (*this), boost::bind (&TriggerUI::trigger_changed, this, _1), gui_context());
-	tref.box->PropertyChanged.connect (trigger_connections, invalidator (*this), boost::bind (&TriggerUI::trigger_changed, this, _1), gui_context ());
+	trigger()->PropertyChanged.connect (trigger_connections, invalidator (*this), std::bind (&TriggerUI::trigger_changed, this, _1), gui_context());
+	tref.box()->PropertyChanged.connect (trigger_connections, invalidator (*this), std::bind (&TriggerUI::trigger_changed, this, _1), gui_context ());
 
-	tref.box->TriggerSwapped.connect (trigger_swap_connection, invalidator (*this), boost::bind (&TriggerUI::trigger_swap, this, _1), gui_context ());
+	tref.box()->TriggerSwapped.connect (trigger_swap_connection, invalidator (*this), std::bind (&TriggerUI::trigger_swap, this, _1), gui_context ());
+
+	if (_color_dialog) {
+		if (trigger()) {
+			Gdk::Color c = Gtkmm2ext::gdk_color_from_rgba(trigger()->color());
+			_color_dialog->get_color_selection()->set_previous_color (c);
+			_color_dialog->get_color_selection()->set_current_color (c);
+		} else {
+			delete _color_dialog;
+			_color_dialog = nullptr;
+		}
+	}
 
 	on_trigger_set();  //derived classes can do initialization here
 }
