@@ -19,8 +19,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef __ardour_midi_track_h__
-#define __ardour_midi_track_h__
+#pragma once
 
 #include "ardour/midi_channel_filter.h"
 #include "ardour/midi_ring_buffer.h"
@@ -94,13 +93,16 @@ public:
 	NoteMode note_mode() const { return _note_mode; }
 	void set_note_mode (NoteMode m);
 
+	bool chase_notes() const { return _chase_notes; }
+	void set_chase_notes (bool yn);
+
 	std::string describe_parameter (Evoral::Parameter param);
 
 	bool step_editing() const { return _step_editing; }
 	void set_step_editing (bool yn);
 	MidiRingBuffer<samplepos_t>& step_edit_ring_buffer() { return _step_edit_ring_buffer; }
 
-	PBD::Signal1<void,bool> StepEditStatusChange;
+	PBD::Signal<void(bool)> StepEditStatusChange;
 
 	std::shared_ptr<SMFSource> write_source (uint32_t n = 0);
 
@@ -122,7 +124,7 @@ public:
 
 	std::shared_ptr<MidiPlaylist> midi_playlist ();
 
-	PBD::Signal1<void, std::weak_ptr<MidiSource> > DataRecorded;
+	PBD::Signal<void(std::weak_ptr<MidiSource> )> DataRecorded;
 	std::shared_ptr<MidiBuffer> get_gui_feed_buffer () const;
 
 	MonitorState monitoring_state () const;
@@ -135,7 +137,7 @@ public:
 
 	void set_input_active (bool);
 	bool input_active () const;
-	PBD::Signal0<void> InputActiveChanged;
+	PBD::Signal<void()> InputActiveChanged;
 
 	void set_restore_pgm_on_load (bool yn);
 	bool restore_pgm_on_load () const;
@@ -143,6 +145,8 @@ public:
 	void realtime_handle_transport_stopped ();
 	void region_edited (std::shared_ptr<Region>);
 
+	int last_seen_external_midi_note () const { return _last_seen_external_midi_note; }
+
 protected:
 
 	XMLNode& state (bool save_template) const;
@@ -153,6 +157,8 @@ protected:
 	void snapshot_out_of_band_data (samplecnt_t nframes);
 	void write_out_of_band_data (BufferSet& bufs, samplecnt_t /* nframes */) const;
 
+	void input_change_handler (IOChange, void *src);
+
 
 private:
 	MidiRingBuffer<samplepos_t> _immediate_events;
@@ -166,6 +172,8 @@ private:
 	bool                        _restore_pgm_on_load;
 	MidiChannelFilter           _playback_filter;
 	MidiChannelFilter           _capture_filter;
+	int                         _last_seen_external_midi_note;
+	bool                        _chase_notes;
 
 	std::shared_ptr<VelocityControl>  _velocity_control;
 
@@ -186,8 +194,9 @@ private:
 
 	void playlist_contents_changed ();
 	PBD::ScopedConnection playlist_content_change_connection;
+
+	PBD::ScopedConnectionList note_connections;
+	void note_on_handler (int);
 };
 
 } /* namespace ARDOUR*/
-
-#endif /* __ardour_midi_track_h__ */
