@@ -29,8 +29,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef __ardour_session_h__
-#define __ardour_session_h__
+#pragma once
 
 #ifdef WAF_BUILD
 #include "libardour-config.h"
@@ -48,7 +47,6 @@
 #include <stdint.h>
 
 #include <boost/dynamic_bitset.hpp>
-#include <boost/scoped_ptr.hpp>
 #include <glibmm/threads.h>
 
 #include <ltc.h>
@@ -57,10 +55,12 @@
 #include "pbd/error.h"
 #include "pbd/event_loop.h"
 #include "pbd/file_archive.h"
+#include "pbd/history_owner.h"
 #include "pbd/rcu.h"
 #include "pbd/statefuldestructible.h"
 #include "pbd/signals.h"
 #include "pbd/undo.h"
+#include "pbd/uuid.h"
 
 #ifdef USE_TLSF
 #  include "pbd/tlsf.h"
@@ -203,17 +203,14 @@ private:
 };
 
 /** Ardour Session */
-class LIBARDOUR_API Session : public PBD::StatefulDestructible, public PBD::ScopedConnectionList, public SessionEventManager, public TransportAPI, public Temporal::TimeDomainProvider
+class LIBARDOUR_API Session : public PBD::HistoryOwner,
+                              public PBD::StatefulDestructible,
+                              public PBD::ScopedConnectionList,
+                              public SessionEventManager,
+                              public TransportAPI,
+                              public Temporal::TimeDomainProvider
 {
-private:
-
 public:
-	enum RecordState {
-		Disabled = 0,
-		Enabled = 1,
-		Recording = 2
-	};
-
 	/* a new session might have non-empty mix_template, an existing session should always have an empty one.
 	   the bus profile can be null if no master out bus is required.
 	*/
@@ -237,6 +234,7 @@ public:
 	 * internal, not user-visible IDs */
 	static unsigned int next_name_id ();
 
+	std::string uuid() const { return _uuid.to_s(); }
 	std::string path() const { return _path; }
 	std::string name() const { return _name; }
 	std::string snap_name() const { return _current_snapshot_name; }
@@ -264,14 +262,14 @@ public:
 
 	void end_unnamed_status () const;
 
-	PBD::Signal0<void> DirtyChanged;
+	PBD::Signal<void()> DirtyChanged;
 
 	const SessionDirectory& session_directory () const { return *(_session_dir.get()); }
 
-	static PBD::Signal1<void,std::string> Dialog;
+	static PBD::Signal<void(std::string)> Dialog;
 
-	PBD::Signal0<void> BatchUpdateStart;
-	PBD::Signal0<void> BatchUpdateEnd;
+	PBD::Signal<void()> BatchUpdateStart;
+	PBD::Signal<void()> BatchUpdateEnd;
 
 	int ensure_subdirs ();
 
@@ -301,7 +299,7 @@ public:
 	std::vector<std::string> get_paths_for_new_sources (bool allow_replacing, const std::string& import_file_path,
 	                                                    uint32_t channels, std::vector<std::string> const & smf_track_names, bool use_smf_file_names);
 
-	int bring_all_sources_into_session (boost::function<void(uint32_t,uint32_t,std::string)> callback);
+	int bring_all_sources_into_session (std::function<void(uint32_t,uint32_t,std::string)> callback);
 
 	void process (pframes_t nframes);
 
@@ -361,6 +359,7 @@ public:
 	uint32_t ntracks () const;
 	uint32_t naudiotracks () const;
 	uint32_t nbusses () const;
+	bool     empty () const;
 
 	bool plot_process_graph (std::string const& file_name) const;
 
@@ -386,6 +385,7 @@ public:
 	bool io_name_is_legal (const std::string&) const;
 	std::shared_ptr<Route> route_by_name (std::string) const;
 	std::shared_ptr<Route> route_by_id (PBD::ID) const;
+	std::shared_ptr<Stripable> stripable_by_name (std::string) const;
 	std::shared_ptr<Stripable> stripable_by_id (PBD::ID) const;
 	std::shared_ptr<Stripable> get_remote_nth_stripable (PresentationInfo::order_t n, PresentationInfo::Flag) const;
 	std::shared_ptr<Route> get_remote_nth_route (PresentationInfo::order_t n) const;
@@ -404,7 +404,7 @@ public:
 	}
 
 	RecordState record_status() const {
-		return _record_status.load(); 
+		return _record_status.load();
 	}
 
 	bool actively_recording () const {
@@ -420,24 +420,27 @@ public:
 
 	void maybe_write_autosave ();
 
-	PBD::Signal1<void, int> SurroundObjectCountChanged;
+	PBD::Signal<void(int)> SurroundObjectCountChanged;
 
 	/* Emitted when all i/o connections are complete */
 
-	PBD::Signal0<void> IOConnectionsComplete;
+	PBD::Signal<void()> IOConnectionsComplete;
 
 	/* Record status signals */
 
-	PBD::Signal0<void> RecordStateChanged; /* signals changes in recording state (i.e. are we recording) */
+	PBD::Signal<void()> RecordStateChanged; /* signals changes in recording state (i.e. are we recording) */
 	/* XXX may 2015: paul says: it isn't clear to me that this has semantics that cannot be inferrred
 	   from the previous signal and session state.
 	*/
-	PBD::Signal0<void> RecordArmStateChanged; /* signals changes in recording arming */
+	PBD::Signal<void()> RecordArmStateChanged; /* signals changes in recording arming */
 
-	PBD::Signal0<void> UpdateRouteRecordState; /* signals potential change in route recording arming */
+	PBD::Signal<void()> UpdateRouteRecordState; /* signals potential change in route recording arming */
+
+	PBD::Signal<void()> RecordPassCompleted;
+	PBD::Signal<void()> ClearedLastCaptureSources;
 
 	/* Emited when session is loaded */
-	PBD::Signal0<void> SessionLoaded;
+	PBD::Signal<void()> SessionLoaded;
 
 	/* Transport mechanism signals */
 
@@ -448,38 +451,42 @@ public:
 	 *  - start (from the process thread)
 	 *  - engine halted
 	 */
-	PBD::Signal0<void> TransportStateChange;
+	PBD::Signal<void()> TransportStateChange;
 
-	PBD::Signal1<void,samplepos_t> PositionChanged; /* sent after any non-sequential motion */
-	PBD::Signal1<void,samplepos_t> Xrun;
-	PBD::Signal0<void> TransportLooped;
+	PBD::Signal<void(samplepos_t)> PositionChanged; /* sent after any non-sequential motion */
+	PBD::Signal<void(samplepos_t)> Xrun;
+	PBD::Signal<void()> TransportLooped;
 
 	/** emitted when a locate has occurred */
-	PBD::Signal0<void> Located;
+	PBD::Signal<void()> Located;
 
-	PBD::Signal1<void,RouteList&> RouteAdded;
+	PBD::Signal<void(RouteList&)> RouteAdded;
+	/* This is emitted after one or more routes are added that are actually
+	   Tracks with an instrument plugin.
+	*/
+	PBD::Signal<void(RouteList&)> InstrumentRouteAdded;
 	/** Emitted when a property of one of our route groups changes.
 	 *  The parameter is the RouteGroup that has changed.
 	 */
-	PBD::Signal1<void, RouteGroup *> RouteGroupPropertyChanged;
+	PBD::Signal<void(std::shared_ptr<RouteGroup>)> RouteGroupPropertyChanged;
 	/** Emitted when a route is added to one of our route groups.
 	 *  First parameter is the RouteGroup, second is the route.
 	 */
-	PBD::Signal2<void, RouteGroup *, std::weak_ptr<Route> > RouteAddedToRouteGroup;
+	PBD::Signal<void(std::shared_ptr<RouteGroup>, std::weak_ptr<Route> )> RouteAddedToRouteGroup;
 	/** Emitted when a route is removed from one of our route groups.
 	 *  First parameter is the RouteGroup, second is the route.
 	 */
-	PBD::Signal2<void, RouteGroup *, std::weak_ptr<Route> > RouteRemovedFromRouteGroup;
+	PBD::Signal<void(std::shared_ptr<RouteGroup>, std::weak_ptr<Route> )> RouteRemovedFromRouteGroup;
 
 	/** Emitted when a foldback send is created or deleted
 	 */
-	PBD::Signal0<void> FBSendsChanged;
+	PBD::Signal<void()> FBSendsChanged;
 
 	/* Step Editing status changed */
-	PBD::Signal1<void,bool> StepEditStatusChange;
+	PBD::Signal<void(bool)> StepEditStatusChange;
 
 	/* Timecode state signals */
-	PBD::Signal0<void> MtcOrLtcInputPortChanged;
+	PBD::Signal<void()> MtcOrLtcInputPortChanged;
 
 	void queue_event (SessionEvent*);
 
@@ -489,6 +496,8 @@ public:
 	void request_stop (bool abort = false, bool clear_state = false, TransportRequestSource origin = TRS_UI);
 	void request_locate (samplepos_t sample, bool force = false, LocateTransportDisposition ltd = RollIfAppropriate, TransportRequestSource origin = TRS_UI);
 
+	bool request_locate_to_mark (std::string const&, LocateTransportDisposition ltd = RollIfAppropriate, TransportRequestSource origin = TRS_UI);
+
 	void request_play_loop (bool yn, bool leave_rolling = false);
 	bool get_play_loop () const { return play_loop; }
 
@@ -545,9 +554,9 @@ public:
 
 	Locations *locations() { return _locations; }
 
-	PBD::Signal1<void,Location*>    auto_loop_location_changed;
-	PBD::Signal1<void,Location*>    auto_punch_location_changed;
-	PBD::Signal0<void>              locations_modified;
+	PBD::Signal<void(Location*)>    auto_loop_location_changed;
+	PBD::Signal<void(Location*)>    auto_punch_location_changed;
+	PBD::Signal<void()>              locations_modified;
 
 	void set_auto_punch_location (Location *);
 	void set_auto_loop_location (Location *);
@@ -563,7 +572,7 @@ public:
 	samplecnt_t worst_latency_preroll () const;
 	samplecnt_t worst_latency_preroll_buffer_size_ceil () const;
 
-	PBD::Signal1<void, bool> LatencyUpdated;
+	PBD::Signal<void(bool)> LatencyUpdated;
 
 	struct SaveAs {
 		std::string new_parent_folder;  /* parent folder where new session folder will be created */
@@ -586,7 +595,7 @@ public:
 		 * Handler should return true for save-as to continue, or false
 		 * to stop (and remove all evidence of partial save-as).
 		 */
-		PBD::Signal3<bool,float,int64_t,int64_t> Progress;
+		PBD::Signal<bool(float,int64_t,int64_t)> Progress;
 
 		/* if save_as() returns non-zero, this string will indicate the reason why.
 		 */
@@ -635,8 +644,8 @@ public:
 	void set_nsm_state (bool state) { _under_nsm_control = state; }
 	bool save_default_options ();
 
-	PBD::Signal1<void,std::string> StateSaved;
-	PBD::Signal0<void> StateReady;
+	PBD::Signal<void(std::string)> StateSaved;
+	PBD::Signal<void()> StateReady;
 
 	/* emitted when session needs to be saved due to some internal
 	 * event or condition (i.e. not in response to a user request).
@@ -646,17 +655,49 @@ public:
 	 *
 	 * Argument is the snapshot name to use when saving.
 	 */
-	PBD::Signal1<void,std::string> SaveSessionRequested;
+	PBD::Signal<void(std::string)> SaveSessionRequested;
 
 	/* emitted during a session save to allow other entities to add state, via
 	 * extra XML, to the session state
 	 */
-	PBD::Signal0<void> SessionSaveUnderway;
+	PBD::Signal<void()> SessionSaveUnderway;
 
 	std::vector<std::string> possible_states() const;
 	static std::vector<std::string> possible_states (std::string path);
 
-	bool export_track_state (std::shared_ptr<RouteList> rl, const std::string& path);
+	enum RouteGroupImportMode {
+		IgnoreRouteGroup,
+		UseRouteGroup,
+		CreateRouteGroup
+	};
+
+	struct RouteImportInfo {
+		RouteImportInfo (std::string const& n, PresentationInfo const& p, int mb)
+			: name (n)
+			, pi (p)
+			, mixbus (mb)
+		{}
+
+		std::string      name;
+		PresentationInfo pi;
+		int              mixbus;
+
+		bool operator< (RouteImportInfo const& o) {
+			if (mixbus != o.mixbus) {
+				return mixbus < o.mixbus;
+			}
+			return name < o.name;
+		}
+
+		bool operator== (RouteImportInfo const& o) {
+			return mixbus == o.mixbus && name == o.name;
+		}
+	};
+
+	bool export_route_state (std::shared_ptr<RouteList> rl, const std::string& path, bool with_sources);
+	int  import_route_state (const std::string& path, std::map<PBD::ID, PBD::ID> const&, RouteGroupImportMode rgim = CreateRouteGroup, PBD::Progress* p = 0);
+
+	std::map<PBD::ID, RouteImportInfo> parse_route_state (const std::string& path, bool& match_pbd_id);
 
 	/// The instant xml file is written to the session directory
 	void add_instant_xml (XMLNode&, bool write_to_config = true);
@@ -704,35 +745,49 @@ public:
 		bool _reconfigure_on_delete;
 	};
 
-	RouteGroup* new_route_group (const std::string&);
-	void add_route_group (RouteGroup *);
-	void remove_route_group (RouteGroup* rg) { if (rg) remove_route_group (*rg); }
-	void remove_route_group (RouteGroup&);
-	void reorder_route_groups (std::list<RouteGroup*>);
+	std::shared_ptr<RouteGroup> new_route_group (const std::string&);
+	void add_route_group (std::shared_ptr<RouteGroup>);
+	void remove_route_group (std::shared_ptr<RouteGroup> rg);
+	void reorder_route_groups (RouteGroupList);
 
-	RouteGroup* route_group_by_name (std::string);
-	RouteGroup& all_route_group() const;
+	std::shared_ptr<RouteGroup> route_group_by_name (std::string);
 
-	PBD::Signal1<void,RouteGroup*> route_group_added;
-	PBD::Signal0<void>             route_group_removed;
-	PBD::Signal0<void>             route_groups_reordered;
+	PBD::Signal<void(std::shared_ptr<RouteGroup>)> route_group_added;
+	PBD::Signal<void()>             route_group_removed;
+	PBD::Signal<void()>             route_groups_reordered;
 
-	void foreach_route_group (boost::function<void(RouteGroup*)> f) {
-		for (std::list<RouteGroup *>::iterator i = _route_groups.begin(); i != _route_groups.end(); ++i) {
-			f (*i);
+	void foreach_route_group (std::function<void(std::shared_ptr<RouteGroup>)> f) {
+		for (auto & rg : _route_groups) {
+			f (rg);
 		}
 	}
 
-	std::list<RouteGroup*> const & route_groups () const {
+	RouteGroupList const & route_groups () const {
 		return _route_groups;
 	}
 
 	/* fundamental operations. duh. */
 
-	std::list<std::shared_ptr<AudioTrack> > new_audio_track (
+	AudioTrackList new_audio_track (
 		int input_channels,
 		int output_channels,
-		RouteGroup* route_group,
+		std::shared_ptr<RouteGroup> route_group,
+		uint32_t how_many,
+		std::string name_template,
+		PresentationInfo::order_t order,
+		TrackMode mode = Normal,
+		bool input_auto_connect = true,
+		bool trigger_visibility = false
+		);
+
+	/* Call this repeatedly with different track name templates and finally call add_routes.
+	 * useful for speeding up imports of various kinds that involve lots of tracks */
+	bool new_audio_routes_tracks_bulk (
+		RouteList& routes,
+		AudioTrackList& tracks,
+		int input_channels,
+		int output_channels,
+		std::shared_ptr<RouteGroup> route_group,
 		uint32_t how_many,
 		std::string name_template,
 		PresentationInfo::order_t order,
@@ -745,19 +800,21 @@ public:
 		const ChanCount& input, const ChanCount& output, bool strict_io,
 		std::shared_ptr<PluginInfo> instrument,
 		Plugin::PresetRecord* pset,
-		RouteGroup* route_group, uint32_t how_many, std::string name_template,
+		std::shared_ptr<RouteGroup> route_group, uint32_t how_many, std::string name_template,
 		PresentationInfo::order_t,
 		TrackMode mode,
 		bool input_auto_connect,
 		bool trigger_visibility = false
 		);
 
-	RouteList new_audio_route (int input_channels, int output_channels, RouteGroup* route_group, uint32_t how_many, std::string name_template, PresentationInfo::Flag, PresentationInfo::order_t);
-	RouteList new_midi_route (RouteGroup* route_group, uint32_t how_many, std::string name_template, bool strict_io, std::shared_ptr<PluginInfo> instrument, Plugin::PresetRecord*, PresentationInfo::Flag, PresentationInfo::order_t);
+	RouteList new_audio_route (int input_channels, int output_channels, std::shared_ptr<RouteGroup> route_group, uint32_t how_many, std::string name_template, PresentationInfo::Flag, PresentationInfo::order_t);
+	RouteList new_midi_route (std::shared_ptr<RouteGroup> route_group, uint32_t how_many, std::string name_template, bool strict_io, std::shared_ptr<PluginInfo> instrument, Plugin::PresetRecord*, PresentationInfo::Flag, PresentationInfo::order_t);
 
 	void remove_routes (std::shared_ptr<RouteList>);
 	void remove_route (std::shared_ptr<Route>);
 
+	void add_routes (RouteList&, bool input_auto_connect, bool output_auto_connect, PresentationInfo::order_t);
+
 	void resort_routes ();
 
 	AudioEngine & engine() { return _engine; }
@@ -772,7 +829,7 @@ public:
 	samplepos_t audible_sample (bool* latent_locate = NULL) const;
 	samplepos_t requested_return_sample() const { return _requested_return_sample; }
 	void set_requested_return_sample(samplepos_t return_to);
-	boost::optional<samplepos_t> const & nominal_jack_transport_sample() { return _nominal_jack_transport_sample; }
+	std::optional<samplepos_t> const & nominal_jack_transport_sample() { return _nominal_jack_transport_sample; }
 
 	bool compute_audible_delta (samplepos_t& pos_and_delta) const;
 	samplecnt_t remaining_latency_preroll () const { return _remaining_latency_preroll; }
@@ -803,8 +860,8 @@ public:
 	samplecnt_t convert_to_samples (AnyTime const & position);
 	samplecnt_t any_duration_to_samples (samplepos_t position, AnyTime const & duration);
 
-	static PBD::Signal1<void, samplepos_t> StartTimeChanged;
-	static PBD::Signal1<void, samplepos_t> EndTimeChanged;
+	static PBD::Signal<void(samplepos_t)> StartTimeChanged;
+	static PBD::Signal<void(samplepos_t)> EndTimeChanged;
 
 	void   request_sync_source (std::shared_ptr<TransportMaster>);
 	bool   synced_to_engine() const;
@@ -832,7 +889,7 @@ public:
 
 	bool punch_is_possible () const;
 	bool loop_is_possible () const;
-	PBD::Signal0<void> PunchLoopConstraintChange;
+	PBD::Signal<void()> PunchLoopConstraintChange;
 
 	void maybe_update_tempo_from_midiclock_tempo (float bpm);
 
@@ -860,8 +917,8 @@ public:
 
 	int start_audio_export (samplepos_t position, bool realtime = false, bool region_export = false);
 
-	PBD::Signal1<int, samplecnt_t> ProcessExport;
-	static PBD::Signal4<void, std::string, std::string, bool, samplepos_t> Exported;
+	PBD::Signal<int(samplecnt_t)> ProcessExport;
+	static PBD::Signal<void(std::string, std::string, bool, samplepos_t)> Exported;
 
 	void add_source (std::shared_ptr<Source>);
 	void remove_source (std::weak_ptr<Source>, bool drop_references = true);
@@ -872,35 +929,39 @@ public:
 	int  cleanup_sources (CleanupReport&);
 	int  cleanup_trash_sources (CleanupReport&);
 
+	void close_all_sources ();
+
 	int destroy_sources (std::list<std::shared_ptr<Source> > const&);
 
 	int remove_last_capture ();
-	void get_last_capture_sources (std::list<std::shared_ptr<Source> >&);
+	bool have_last_capture_sources () const;
+	void last_capture_sources (std::list<std::shared_ptr<Source> >&) const;
+	void reset_last_capture_sources ();
 
 	/** handlers should return -1 for "stop cleanup",
 	    0 for "yes, delete this playlist",
 	    1 for "no, don't delete this playlist".
 	*/
-	static PBD::Signal1<int,std::shared_ptr<Playlist> >  AskAboutPlaylistDeletion;
+	static PBD::Signal<int(std::shared_ptr<Playlist> )>  AskAboutPlaylistDeletion;
 
 	/** handlers should return 0 for "ignore the rate mismatch",
 	    !0 for "do not use this session"
 	*/
-	static PBD::Signal2<int, samplecnt_t, samplecnt_t> AskAboutSampleRateMismatch;
+	static PBD::Signal<int(samplecnt_t, samplecnt_t)> AskAboutSampleRateMismatch;
 
 	/** non interactive message */
-	static PBD::Signal2<void, samplecnt_t, samplecnt_t> NotifyAboutSampleRateMismatch;
+	static PBD::Signal<void(samplecnt_t, samplecnt_t)> NotifyAboutSampleRateMismatch;
 
 	/** handlers should return !0 for use pending state, 0 for ignore it.
 	 */
-	static PBD::Signal0<int> AskAboutPendingState;
+	static PBD::Signal<int()> AskAboutPendingState;
 
 	/** after loading a session, once all ports have been created and connected
 	 * signal is emitted to let objects that need to do some housekeeping
 	 * post-connect.
 	 */
 
-	static PBD::Signal0<void> AfterConnect;
+	static PBD::Signal<void()> AfterConnect;
 
 	std::shared_ptr<AudioFileSource> create_audio_source_for_session (
 		size_t, std::string const &, uint32_t);
@@ -926,7 +987,7 @@ public:
 	void cancel_audition ();
 	bool is_auditioning () const;
 
-	PBD::Signal1<void,bool> AuditionActive;
+	PBD::Signal<void(bool)> AuditionActive;
 
 	/* session script */
 	void register_lua_function (const std::string&, const std::string&, const LuaScriptParamList&);
@@ -935,10 +996,10 @@ public:
 	uint32_t registered_lua_function_count () const { return _n_lua_scripts; }
 	void scripts_changed (); // called from lua, updates _n_lua_scripts
 
-	PBD::Signal0<void> LuaScriptsChanged;
+	PBD::Signal<void()> LuaScriptsChanged;
 
 	/* I/O Plugin */
-	PBD::Signal0<void> IOPluginsChanged;
+	PBD::Signal<void()> IOPluginsChanged;
 
 	void load_io_plugin (std::shared_ptr<IOPlug>);
 	bool unload_io_plugin (std::shared_ptr<IOPlug>);
@@ -989,15 +1050,15 @@ public:
 
 	void set_exclusive_input_active (std::shared_ptr<RouteList> rt, bool onoff, bool flip_others = false);
 
-	PBD::Signal1<void,bool> SoloActive;
-	PBD::Signal0<void> SoloChanged;
-	PBD::Signal0<void> MuteChanged;
-	PBD::Signal0<void> IsolatedChanged;
-	PBD::Signal0<void> MonitorChanged;
-	PBD::Signal0<void> MonitorBusAddedOrRemoved;
-	PBD::Signal0<void> SurroundMasterAddedOrRemoved;
+	PBD::Signal<void(bool)> SoloActive;
+	PBD::Signal<void()> SoloChanged;
+	PBD::Signal<void()> MuteChanged;
+	PBD::Signal<void()> IsolatedChanged;
+	PBD::Signal<void()> MonitorChanged;
+	PBD::Signal<void()> MonitorBusAddedOrRemoved;
+	PBD::Signal<void()> SurroundMasterAddedOrRemoved;
 
-	PBD::Signal0<void> session_routes_reconnected;
+	PBD::Signal<void()> session_routes_reconnected;
 
 	/* monitor/master out */
 	int add_master_bus (ChanCount const&);
@@ -1055,7 +1116,7 @@ public:
 
 	/* s/w "RAID" management */
 
-	boost::optional<samplecnt_t> available_capture_duration();
+	std::optional<samplecnt_t> available_capture_duration();
 
 	/* I/O bundles */
 
@@ -1063,7 +1124,7 @@ public:
 	void remove_bundle (std::shared_ptr<Bundle>);
 	std::shared_ptr<Bundle> bundle_by_name (std::string) const;
 
-	PBD::Signal0<void> BundleAddedOrRemoved;
+	PBD::Signal<void()> BundleAddedOrRemoved;
 
 	void midi_panic ();
 
@@ -1085,70 +1146,6 @@ public:
 	std::string next_undo() const { return _history.next_undo(); }
 	std::string next_redo() const { return _history.next_redo(); }
 
-	/** begin collecting undo information
-	 *
-	 * This call must always be followed by either
-	 * begin_reversible_command() or commit_reversible_command()
-	 *
-	 * @param cmd_name human readable name for the undo operation
-	 */
-	void begin_reversible_command (const std::string& cmd_name);
-	void begin_reversible_command (GQuark);
-	/** abort an open undo command
-	 * This must only be called after begin_reversible_command ()
-	 */
-	void abort_reversible_command ();
-	/** finalize an undo command and commit pending transactions
-	 *
-	 * This must only be called after begin_reversible_command ()
-	 * @param cmd (additional) command to add
-	 */
-	void commit_reversible_command (PBD::Command* cmd = 0);
-
-	void add_command (PBD::Command *const cmd);
-
-	/** create an StatefulDiffCommand from the given object and add it to the stack.
-	 *
-	 * This function must only be called after  begin_reversible_command.
-	 * Failing to do so may lead to a crash.
-	 *
-	 * @param sfd the object to diff
-	 * @returns the allocated StatefulDiffCommand (already added via add_command)
-	 */
-	PBD::StatefulDiffCommand* add_stateful_diff_command (std::shared_ptr<PBD::StatefulDestructible> sfd);
-
-	/** @return The list of operations that are currently in progress */
-	std::list<GQuark> const & current_operations () {
-		return _current_trans_quarks;
-	}
-
-	bool operation_in_progress (GQuark) const;
-
-	/**
-	 * Test if any undo commands were added since the
-	 * call to begin_reversible_command ()
-	 *
-	 * This is useful to determine if an undoable
-	 * action was performed before adding additional
-	 * information (e.g. selection changes) to the
-	 * undo transaction.
-	 *
-	 * @return true if undo operation is valid but empty
-	 */
-	bool collected_undo_commands () const {
-		return _current_trans && !_current_trans->empty ();
-	}
-
-	PBD::UndoTransaction* current_reversible_command() { return _current_trans; }
-
-	/**
-	 * Abort reversible command IFF no undo changes
-	 * have been collected.
-	 * @return true if undo operation was aborted.
-	 */
-	bool abort_empty_reversible_command ();
-
-	void add_commands (std::vector<PBD::Command*> const & cmds);
 
 	std::map<PBD::ID,PBD::StatefulDestructible*> registry;
 
@@ -1216,7 +1213,7 @@ public:
 		float opt
 		);
 
-	static PBD::Signal0<void> SendFeedback;
+	static PBD::Signal<void()> SendFeedback;
 
 	/* Speakers */
 
@@ -1303,12 +1300,12 @@ public:
 	    processing path.  Until it is fixed (by the user) some (unspecified)
 	    routes will not be run.
 	*/
-	static PBD::Signal0<void> FeedbackDetected;
+	static PBD::Signal<void()> FeedbackDetected;
 
 	/** Emitted when a graph sort has successfully completed, which means
 	    that it has no feedback cycles.
 	*/
-	static PBD::Signal0<void> SuccessfulGraphSort;
+	static PBD::Signal<void()> SuccessfulGraphSort;
 
 	/* handlers can return an integer value:
 	   0: config.set_audio_search_path() or config.set_midi_search_path() was used
@@ -1319,19 +1316,19 @@ public:
 	   -1: just mark this one missing
 	   any other value: as -1
 	*/
-	static PBD::Signal3<int,Session*,std::string,DataType> MissingFile;
+	static PBD::Signal<int(Session*,std::string,DataType)> MissingFile;
 
 	void set_missing_file_replacement (const std::string& mfr) {
 		_missing_file_replacement = mfr;
 	}
 
 	/** Emitted when the session wants Ardour to quit */
-	static PBD::Signal0<void> Quit;
+	static PBD::Signal<void()> Quit;
 
 	/** Emitted when Ardour is asked to load a session in an older session
 	 * format, and makes a backup copy.
 	 */
-	static PBD::Signal2<void,std::string,std::string> VersionMismatch;
+	static PBD::Signal<void(std::string,std::string)> VersionMismatch;
 
 	SceneChanger* scene_changer() const { return _scene_changer; }
 
@@ -1372,7 +1369,7 @@ public:
 	void import_pt_rest (PTFFormat& ptf);
 	bool import_sndfile_as_region (std::string path, SrcQuality quality, timepos_t& pos, SourceList& sources, ImportStatus& status, uint32_t current, uint32_t total);
 
-	struct ptflookup {
+	typedef struct ptflookup {
 		uint16_t index1;
 		uint16_t index2;
 		PBD::ID  id;
@@ -1380,9 +1377,8 @@ public:
 		bool operator ==(const struct ptflookup& other) {
 			return (this->index1 == other.index1);
 		}
-	};
-	std::vector<struct ptflookup> ptfwavpair;
-	SourceList pt_imported_sources;
+	} PtfLookup;
+	std::vector<PtfLookup> ptfregpair;
 
 	enum TimingTypes {
 		OverallProcess = 0,
@@ -1405,10 +1401,16 @@ public:
 	bool bang_trigger_at(int32_t route_index, int32_t row_index, float velocity = 1.0);
 	bool unbang_trigger_at(int32_t route_index, int32_t row_index);
 	void clear_cue (int row_index);
+	std::shared_ptr<TriggerBox> armed_triggerbox () const;
+	std::shared_ptr<TriggerBox> rec_enabled_triggerbox () const;
 
 	void start_domain_bounce (Temporal::DomainBounceInfo&);
 	void finish_domain_bounce (Temporal::DomainBounceInfo&);
 
+	AnyTime global_quantization() const { return _global_quantization; }
+	void set_global_quantization (AnyTime const &);
+	PBD::Signal<void()> QuantizationChanged;
+
 protected:
 	friend class AudioEngine;
 	void set_block_size (pframes_t nframes);
@@ -1503,8 +1505,8 @@ private:
 	void add_surround_master ();
 	void remove_surround_master ();
 
-	boost::optional<bool> _vapor_available;
-	boost::optional<bool> _vapor_exportable;
+	std::optional<bool> _vapor_available;
+	std::optional<bool> _vapor_exportable;
 
 	void update_latency (bool playback);
 	void set_owned_port_public_latency (bool playback);
@@ -1608,6 +1610,7 @@ private:
 	bool maybe_stop (samplepos_t limit);
 	bool maybe_sync_start (pframes_t &);
 
+	PBD::UUID               _uuid;
 	std::string             _path;
 	std::string             _name;
 	bool                    _is_new;
@@ -1616,11 +1619,11 @@ private:
 	 *  know when to send full MTC messages every so often.
 	 */
 	pframes_t               _pframes_since_last_mtc;
-	bool                     play_loop;
+	std::atomic<bool>        play_loop;
 	bool                     loop_changing;
 	samplepos_t              last_loopend;
 
-	boost::scoped_ptr<SessionDirectory> _session_dir;
+	const std::unique_ptr<SessionDirectory> _session_dir;
 
 	void hookup_io ();
 	void graph_reordered (bool called_from_backend);
@@ -1841,10 +1844,10 @@ private:
 	void mmc_shuttle (MIDI::MachineControl &mmc, float speed, bool forw);
 	void mmc_record_enable (MIDI::MachineControl &mmc, size_t track, bool enabled);
 
-	struct timeval last_mmc_step;
-	double step_speed;
+	int64_t _last_mmc_step;
+	double  step_speed;
 
-	typedef boost::function<bool()> MidiTimeoutCallback;
+	typedef std::function<bool()> MidiTimeoutCallback;
 	typedef std::list<MidiTimeoutCallback> MidiTimeoutList;
 
 	MidiTimeoutList midi_timeouts;
@@ -1963,14 +1966,13 @@ private:
 
 	int load_route_groups (const XMLNode&, int);
 
-	std::list<RouteGroup *> _route_groups;
-	RouteGroup*             _all_route_group;
+	RouteGroupList _route_groups;
+	void route_group_emptied (std::shared_ptr<RouteGroup>);
 
 	/* routes stuff */
 
 	SerializedRCUManager<RouteList>  routes;
 
-	void add_routes (RouteList&, bool input_auto_connect, bool output_auto_connect, PresentationInfo::order_t);
 	void add_routes_inner (RouteList&, bool input_auto_connect, bool output_auto_connect, PresentationInfo::order_t);
 	bool _adding_routes_in_progress;
 	bool _reconnecting_routes_in_progress;
@@ -2012,6 +2014,24 @@ private:
 	void listen_position_changed ();
 	void solo_control_mode_changed ();
 
+	bool _solo_change_in_progress;
+
+	struct SoloChange {
+		SoloChange (bool s, PBD::Controllable::GroupControlDisposition g, std::weak_ptr<Route> r)
+			: self_solo_changed (s)
+			, gcd (g)
+			, route (r)
+		{}
+		bool                                       self_solo_changed;
+		PBD::Controllable::GroupControlDisposition gcd;
+		std::weak_ptr<Route>                       route;
+	};
+#ifdef _MSC_VER
+	std::vector<SoloChange> _solo_change_queue;
+#else
+	std::vector<SoloChange, PBD::StackAllocator<SoloChange, 8>> _solo_change_queue;
+#endif
+
 	/* REGION MANAGEMENT */
 
 	mutable Glib::Threads::Mutex region_lock;
@@ -2019,9 +2039,9 @@ private:
 	int load_regions (const XMLNode& node);
 	int load_compounds (const XMLNode& node);
 
-	void route_added_to_route_group (RouteGroup *, std::weak_ptr<Route>);
-	void route_removed_from_route_group (RouteGroup *, std::weak_ptr<Route>);
-	void route_group_property_changed (RouteGroup *);
+	void route_added_to_route_group (std::shared_ptr<RouteGroup>, std::weak_ptr<Route>);
+	void route_removed_from_route_group (std::shared_ptr<RouteGroup>, std::weak_ptr<Route>);
+	void route_group_property_changed (std::weak_ptr<RouteGroup>);
 
 	/* SOURCES */
 
@@ -2030,12 +2050,12 @@ private:
 public:
 
 	/* Emited when a new source is added to the session */
-	PBD::Signal1< void, std::weak_ptr<Source> > SourceAdded;
-	PBD::Signal1< void, std::weak_ptr<Source> > SourceRemoved;
+	PBD::Signal< void(std::weak_ptr<Source> )> SourceAdded;
+	PBD::Signal< void(std::weak_ptr<Source> )> SourceRemoved;
 
 	typedef std::map<PBD::ID,std::shared_ptr<Source> > SourceMap;
 
-	void foreach_source (boost::function<void( std::shared_ptr<Source> )> f) {
+	void foreach_source (std::function<void( std::shared_ptr<Source> )> f) {
 		Glib::Threads::Mutex::Lock ls (source_lock);
 		for (SourceMap::iterator i = sources.begin(); i != sources.end(); ++i) {
 			f ( (*i).second );
@@ -2046,7 +2066,7 @@ public:
 	ARDOUR::CueMarkers pending_source_markers; // source markers created while recording
 
 private:
-	void reset_write_sources (bool mark_write_complete, bool force = false);
+	void reset_write_sources (bool mark_write_complete);
 	SourceMap sources;
 
 	int load_sources (const XMLNode& node);
@@ -2141,14 +2161,7 @@ private:
 	XMLNode* _bundle_xml_node;
 	int load_bundles (XMLNode const &);
 
-	PBD::UndoHistory      _history;
-	/** current undo transaction, or 0 */
-	PBD::UndoTransaction* _current_trans;
-	/** GQuarks to describe the reversible commands that are currently in progress.
-	 *  These may be nested, in which case more recently-started commands are toward
-	 *  the front of the list.
-	 */
-	std::list<GQuark> _current_trans_quarks;
+	mutable XMLNode* _engine_state;
 
 	int  backend_sync_callback (TransportState, samplepos_t);
 
@@ -2266,6 +2279,8 @@ private:
 
 	mutable bool have_looped; ///< Used in \ref audible_sample
 
+	bool roll_started_loop;
+
 	void update_route_record_state ();
 	std::atomic<int> _have_rec_enabled_track;
 	std::atomic<int> _have_rec_disabled_track;
@@ -2277,7 +2292,7 @@ private:
 		get_rt_event (std::shared_ptr<RouteList const> rl, T targ, SessionEvent::RTeventCallback after, PBD::Controllable::GroupControlDisposition group_override,
 		              void (Session::*method) (std::shared_ptr<RouteList const>, T, PBD::Controllable::GroupControlDisposition)) {
 		SessionEvent* ev = new SessionEvent (SessionEvent::RealTimeOperation, SessionEvent::Add, SessionEvent::Immediate, 0, 0.0);
-		ev->rt_slot = boost::bind (method, this, rl, targ, group_override);
+		ev->rt_slot = std::bind (method, this, rl, targ, group_override);
 		ev->rt_return = after;
 		ev->event_loop = PBD::EventLoop::get_event_loop_for_thread ();
 
@@ -2289,7 +2304,7 @@ private:
 		get_rt_event (std::shared_ptr<RouteList const> rl, T1 t1arg, T2 t2arg, SessionEvent::RTeventCallback after, PBD::Controllable::GroupControlDisposition group_override,
 		              void (Session::*method) (std::shared_ptr<RouteList const>, T1, T2, PBD::Controllable::GroupControlDisposition)) {
 		SessionEvent* ev = new SessionEvent (SessionEvent::RealTimeOperation, SessionEvent::Add, SessionEvent::Immediate, 0, 0.0);
-		ev->rt_slot = boost::bind (method, this, rl, t1arg, t2arg, group_override);
+		ev->rt_slot = std::bind (method, this, rl, t1arg, t2arg, group_override);
 		ev->rt_return = after;
 		ev->event_loop = PBD::EventLoop::get_event_loop_for_thread ();
 
@@ -2299,7 +2314,7 @@ private:
 	/* specialized version realtime "apply to set of controls" operations */
 	SessionEvent* get_rt_event (std::shared_ptr<WeakAutomationControlList> cl, double arg, PBD::Controllable::GroupControlDisposition group_override) {
 		SessionEvent* ev = new SessionEvent (SessionEvent::RealTimeOperation, SessionEvent::Add, SessionEvent::Immediate, 0, 0.0);
-		ev->rt_slot = boost::bind (&Session::rt_set_controls, this, cl, arg, group_override);
+		ev->rt_slot = std::bind (&Session::rt_set_controls, this, cl, arg, group_override);
 		ev->rt_return = Session::rt_cleanup;
 		ev->event_loop = PBD::EventLoop::get_event_loop_for_thread ();
 
@@ -2366,6 +2381,7 @@ private:
 	void setup_click ();
 	void setup_click_state (const XMLNode*);
 	void setup_bundles ();
+	void setup_bundles_rcu ();
 
 	void port_registry_changed ();
 	void probe_ctrl_surfaces ();
@@ -2394,7 +2410,7 @@ private:
 	CoreSelection* _selection;
 
 	bool _global_locate_pending;
-	boost::optional<samplepos_t> _nominal_jack_transport_sample;
+	std::optional<samplepos_t> _nominal_jack_transport_sample;
 
 	bool _had_destructive_tracks;
 
@@ -2424,9 +2440,10 @@ private:
 	void handle_slots_empty_status (std::weak_ptr<Route> const &);
 
 	void time_domain_changed ();
+
+	AnyTime _global_quantization;
 };
 
 
 } // namespace ARDOUR
 
-#endif /* __ardour_session_h__ */
