@@ -16,8 +16,9 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#ifndef __libardour_trimmable_h__
-#define __libardour_trimmable_h__
+#pragma once
+
+#include "ardour/libardour_visibility.h"
 
 namespace ARDOUR {
 
@@ -51,4 +52,3 @@ class LIBARDOUR_API Trimmable {
 
 }
 
-#endif /* __libardour_trimmable_h__ */
