@@ -17,8 +17,6 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include <boost/smart_ptr/scoped_array.hpp>
-
 #include "pbd/enumwriter.h"
 #include "pbd/memento_command.h"
 #include "pbd/playback_buffer.h"
@@ -48,7 +46,7 @@ using namespace PBD;
 using namespace std;
 
 ARDOUR::samplecnt_t   DiskReader::_chunk_samples = default_chunk_samples ();
-PBD::Signal0<void>    DiskReader::Underrun;
+PBD::Signal<void()>    DiskReader::Underrun;
 thread_local Sample*  DiskReader::_sum_buffer     = 0;
 thread_local Sample*  DiskReader::_mixdown_buffer = 0;
 thread_local gain_t*  DiskReader::_gain_buffer    = 0;
@@ -208,6 +206,37 @@ DiskReader::realtime_locate (bool for_loop_end)
 	}
 }
 
+void
+DiskReader::clear_midi_chase ()
+{
+	_locate_tracker.reset ();
+}
+
+void
+DiskReader::midi_chase (samplepos_t spos)
+{
+	RTMidiBuffer* rtmb = rt_midibuffer();
+	if (!rtmb) {
+		return;
+	}
+
+	_locate_tracker.reset ();
+
+	if (rtmb) {
+		for (size_t n = 0; n < rtmb->size(); ++n) {
+			uint32_t sz;
+			RTMidiBuffer::Item const & item ((*rtmb)[n]);
+			if (item.timestamp > spos) {
+				break;
+			}
+			uint8_t const * buf = rtmb->bytes (item, sz);
+			if (sz == 3) {
+				_locate_tracker.track (buf);
+			}
+		}
+	}
+}
+
 float
 DiskReader::buffer_load () const
 {
@@ -470,7 +499,7 @@ midi:
 
 	/* MIDI data handling */
 
-	const bool no_playlist_modification_pending = !(pending_overwrite () & PlaylistModified);
+	const bool playlist_modification_pending = 0 != (_pending_overwrite.load () & PlaylistModified);
 
 	if (bufs.count ().n_midi ()) {
 		MidiBuffer& dst (bufs.get_midi (0));
@@ -480,16 +509,12 @@ midi:
 			run_must_resolve = false;
 		}
 
-		if (!no_disk_output && !declick_in_progress () && (ms & MonitoringDisk) && !still_locating && no_playlist_modification_pending && speed) {
+		if (!no_disk_output && !declick_in_progress () && (ms & MonitoringDisk) && !still_locating && !playlist_modification_pending && speed) {
 			get_midi_playback (dst, start_sample, end_sample, ms, scratch_bufs, speed, disk_samples_to_consume);
 		}
 	}
 
-	/* decide if we need the butler */
-
-	if (!still_locating && no_playlist_modification_pending) {
-		bool butler_required = false;
-
+	if (!still_locating) {
 		if (speed < 0.0) {
 			playback_sample -= disk_samples_to_consume;
 		} else {
@@ -501,6 +526,11 @@ midi:
 			Temporal::Range loop_range (loc->start (), loc->end ());
 			playback_sample = loop_range.squish (timepos_t (playback_sample)).samples ();
 		}
+	}
+
+	/* decide if we need the butler */
+	if (!still_locating && !playlist_modification_pending) {
+		bool butler_required = false;
 
 		if (_playlists[DataType::AUDIO]) {
 			if (!c->empty ()) {
@@ -723,9 +753,9 @@ DiskReader::overwrite_existing_audio ()
 		chunk2_cnt = to_overwrite - chunk1_cnt;
 	}
 
-	boost::scoped_array<Sample> sum_buffer (new Sample[to_overwrite]);
-	boost::scoped_array<Sample> mixdown_buffer (new Sample[to_overwrite]);
-	boost::scoped_array<float>  gain_buffer (new float[to_overwrite]);
+	std::unique_ptr<Sample[]> sum_buffer (new Sample[to_overwrite]);
+	std::unique_ptr<Sample[]> mixdown_buffer (new Sample[to_overwrite]);
+	std::unique_ptr<float[]>  gain_buffer (new float[to_overwrite]);
 	uint32_t                    n   = 0;
 	bool                        ret = true;
 	samplepos_t                 start = overwrite_sample;
@@ -1136,11 +1166,14 @@ DiskReader::do_refill_with_alloc (bool partial_fill, bool reversed)
 	 * samples would be 1M samples. But we might use 16 or 14 bit samples,
 	 * in which case 4MB is more samples than that. Therefore size this for
 	 * the smallest sample value .. 4MB = 2M samples (16 bit).
+	 *
+	 * Note, we cannot use std::array<> here since stack-size is limited.
 	 */
 
-	boost::scoped_array<Sample> sum_buf (new Sample[2 * 1048576]);
-	boost::scoped_array<Sample> mix_buf (new Sample[2 * 1048576]);
-	boost::scoped_array<float>  gain_buf (new float[2 * 1048576]);
+	constexpr size_t chunksize = 2 * 1048576;
+	std::unique_ptr<Sample[]> sum_buf (new Sample[chunksize]);
+	std::unique_ptr<Sample[]> mix_buf (new Sample[chunksize]);
+	std::unique_ptr<float[]>  gain_buf (new float[chunksize]);
 
 	return refill_audio (sum_buf.get (), mix_buf.get (), gain_buf.get (), (partial_fill ? _chunk_samples : 0), reversed);
 }
@@ -1421,7 +1454,7 @@ DiskReader::playlist_ranges_moved (list<Temporal::RangeMove> const& movements, b
 	}
 
 	/* move processor automation */
-	_track.foreach_processor (boost::bind (&DiskReader::move_processor_automation, this, _1, movements));
+	_track.foreach_processor (std::bind (&DiskReader::move_processor_automation, this, _1, movements));
 }
 
 void
@@ -1502,6 +1535,10 @@ DiskReader::get_midi_playback (MidiBuffer& dst, samplepos_t start_sample, sample
 	if (ms & MonitoringDisk) {
 		/* disk data needed */
 
+		if (Config->get_midi_chase() && _locate_tracker.on()) {
+			_locate_tracker.flush_notes (dst, 0, true);
+		}
+
 		Location* loc = _loop_location;
 
 		if (loc) {
@@ -1625,7 +1662,6 @@ DiskReader::dec_no_disk_output ()
 /* min gain difference for de-click and loop-fadess
  * (-60dB difference to target)
  */
-#define GAIN_COEFF_DELTA (1e-5)
 
 DiskReader::DeclickAmp::DeclickAmp (samplecnt_t sample_rate)
 {
@@ -1993,8 +2029,8 @@ DiskReader::setup_preloop_buffer ()
 	}
 
 	Location*                   loc = _loop_location;
-	boost::scoped_array<Sample> mix_buf (new Sample[loop_fade_length]);
-	boost::scoped_array<Sample> gain_buf (new Sample[loop_fade_length]);
+	std::unique_ptr<Sample[]> mix_buf (new Sample[loop_fade_length]);
+	std::unique_ptr<Sample[]> gain_buf (new Sample[loop_fade_length]);
 	const timepos_t             read_start = timepos_t (loc->start_sample () - loop_declick_out.fade_length);
 	const timecnt_t             read_cnt   = timecnt_t (loop_declick_out.fade_length);
 
