@@ -152,6 +152,14 @@ MidiPort::get_midi_buffer (pframes_t nframes)
 
 			timestamp -= _global_port_buffer_offset;
 
+			if (size == 3) {
+				if (((buf[0] & 0xF0) == 0x90) && (buf[2] != 0)) {
+					NoteOn (buf[1]);
+				} else if (((buf[0] & 0xF0) == 0x80) || (((buf[0] & 0xF0) == 0x90) && (buf[2] == 0))) {
+					NoteOff (buf[1]);
+				}
+			}
+
 			if ((buf[0] & 0xF0) == 0x90 && buf[2] == 0) {
 				/* normalize note on with velocity 0 to proper note off */
 				uint8_t ev[3];
@@ -273,10 +281,13 @@ MidiPort::flush_buffers (pframes_t nframes)
 
 	void* port_buffer = 0;
 
-	if (_resolve_required) {
+	if (_resolve_required && (_global_port_buffer_offset + nframes) > 0) {
 		port_buffer = port_engine.get_buffer (_port_handle, nframes);
-		/* resolve all notes at the start of the buffer */
-		resolve_notes (port_buffer, _global_port_buffer_offset);
+		/* This is called from ARDOUR::PortManager::cycle_end() at which
+		 * point other MIDI data has already been written to the port.
+		 * So we "resolve notes" (panic) at the end of he buffer.
+		 */
+		resolve_notes (port_buffer, _global_port_buffer_offset + nframes -1);
 		_resolve_required = false;
 	}
 
@@ -375,8 +386,11 @@ MidiPort::transport_stopped ()
 }
 
 void
-MidiPort::realtime_locate (bool)
+MidiPort::realtime_locate (bool for_loop_end)
 {
+	if (for_loop_end && !Config->get_midi_panic_when_looping()) {
+		return;
+	}
 	_resolve_required = true;
 }
 
