@@ -90,11 +90,13 @@ MidiTrack::MidiTrack (Session& sess, string name, TrackMode mode)
 	, _step_editing (false)
 	, _input_active (true)
 	, _restore_pgm_on_load (true)
+	, _last_seen_external_midi_note (-1)
+	, _chase_notes (true)
 {
-	_session.SessionLoaded.connect_same_thread (*this, boost::bind (&MidiTrack::restore_controls, this));
+	_session.SessionLoaded.connect_same_thread (*this, std::bind (&MidiTrack::restore_controls, this));
 
-	_playback_filter.ChannelModeChanged.connect_same_thread (*this, boost::bind (&Track::playlist_modified, this));
-	_playback_filter.ChannelMaskChanged.connect_same_thread (*this, boost::bind (&Track::playlist_modified, this));
+	_playback_filter.ChannelModeChanged.connect_same_thread (*this, std::bind (&Track::playlist_modified, this));
+	_playback_filter.ChannelMaskChanged.connect_same_thread (*this, std::bind (&Track::playlist_modified, this));
 }
 
 MidiTrack::~MidiTrack ()
@@ -114,12 +116,12 @@ MidiTrack::init ()
 	_velocity_control.reset (new VelocityControl (_session));
 	add_control (_velocity_control);
 
-	_input->changed.connect_same_thread (*this, boost::bind (&MidiTrack::track_input_active, this, _1, _2));
+	_input->changed.connect_same_thread (*this, std::bind (&MidiTrack::track_input_active, this, _1, _2));
 
 	_disk_writer->set_note_mode (_note_mode);
 	_disk_reader->reset_tracker ();
 
-	_disk_writer->DataRecorded.connect_same_thread (*this, boost::bind (&MidiTrack::data_recorded, this, _1));
+	_disk_writer->DataRecorded.connect_same_thread (*this, std::bind (&MidiTrack::data_recorded, this, _1));
 
 #ifdef HAVE_BEATBOX
 	_beatbox.reset (new BeatBox (_session));
@@ -129,6 +131,18 @@ MidiTrack::init ()
 	return 0;
 }
 
+void
+MidiTrack::set_chase_notes (bool yn)
+{
+	if (_chase_notes != yn) {
+		_chase_notes = yn;
+		if (!yn && _disk_reader) {
+			_disk_reader->clear_midi_chase ();
+		}
+		_session.set_dirty();
+	}
+}
+
 void
 MidiTrack::data_recorded (std::weak_ptr<MidiSource> src)
 {
@@ -206,6 +220,11 @@ MidiTrack::set_state (const XMLNode& node, int version)
 		capture_channel_mode = playback_channel_mode;
 	}
 
+	if (!node.get_property ("chase-notes", _chase_notes)) {
+		/* default is to chase */
+		_chase_notes = true;
+	}
+
 	XMLProperty const * prop;
 
 	unsigned int playback_channel_mask = 0xffff;
@@ -229,7 +248,7 @@ MidiTrack::set_state (const XMLNode& node, int version)
 
 	if (_session.loading ()) {
 		_session.StateReady.connect_same_thread (
-			*this, boost::bind (&MidiTrack::set_state_part_two, this));
+			*this, std::bind (&MidiTrack::set_state_part_two, this));
 	} else {
 		set_state_part_two ();
 	}
@@ -274,6 +293,7 @@ MidiTrack::state(bool save_template) const
 	root.set_property ("step-editing", _step_editing);
 	root.set_property ("input-active", _input_active);
 	root.set_property ("restore-pgm", _restore_pgm_on_load);
+	root.set_property ("chase-notes", _chase_notes);
 
 	for (Controls::const_iterator c = _controls.begin(); c != _controls.end(); ++c) {
 		if (std::dynamic_pointer_cast<MidiTrack::MidiControl>(c->second)) {
@@ -442,6 +462,10 @@ MidiTrack::non_realtime_locate (samplepos_t spos)
 		return;
 	}
 
+	if (_chase_notes && Config->get_midi_chase()) {
+		_disk_reader->midi_chase (spos);
+	}
+
 	Glib::Threads::Mutex::Lock lm (_control_lock, Glib::Threads::TRY_LOCK);
 	if (!lm.locked()) {
 		return;
@@ -490,7 +514,7 @@ MidiTrack::push_midi_input_to_step_edit_ringbuffer (samplecnt_t nframes)
 			   elsewhere
 			*/
 
-			if (ev.is_note_on()) {
+			if (ev.is_note ()) {
 				/* we don't care about the time for this purpose */
 				_step_edit_ring_buffer.write (0, ev.event_type(), ev.size(), ev.buffer());
 			}
@@ -780,7 +804,7 @@ MidiTrack::MidiControl::actually_set_value (double val, PBD::Controllable::Group
 void
 MidiTrack::set_step_editing (bool yn)
 {
-	if (_session.record_status() != Session::Disabled) {
+	if (_session.record_status() != Disabled) {
 		return;
 	}
 
@@ -915,24 +939,18 @@ MidiTrack::act_on_mute ()
 
 	if (muted() || _mute_master->muted_by_others_soloing_at (MuteMaster::AllPoints)) {
 		/* only send messages for channels we are using */
-
-		uint16_t mask = _playback_filter.get_channel_mask();
-
-		for (uint8_t channel = 0; channel <= 0xF; channel++) {
-
-			if ((1<<channel) & mask) {
-
-				DEBUG_TRACE (DEBUG::MidiIO, string_compose ("%1 delivers mute message to channel %2\n", name(), channel+1));
-				uint8_t ev[3] = { ((uint8_t) (MIDI_CMD_CONTROL | channel)), MIDI_CTL_SUSTAIN, 0 };
-				write_immediate_event (Evoral::MIDI_EVENT, 3, ev);
-
-				/* Note we do not send MIDI_CTL_ALL_NOTES_OFF here, since this may
-				   silence notes that came from another non-muted track. */
+		foreach_processor ([this](std::weak_ptr<Processor> p) {
+			std::shared_ptr<Delivery> delivery = std::dynamic_pointer_cast<Delivery> (p.lock());
+			if (delivery) {
+				delivery->set_midi_mute_mask (_playback_filter.get_channel_mask());
 			}
-		}
+		});
 
 		/* Resolve active notes. */
-		_disk_reader->resolve_tracker (_immediate_events, 0);
+
+		if (!the_instrument()) {
+			_disk_reader->resolve_tracker (_immediate_events, 0);
+		}
 	}
 }
 
@@ -985,3 +1003,25 @@ MidiTrack::playlist_contents_changed ()
 
 {
 }
+
+void
+MidiTrack::input_change_handler (IOChange change, void *src)
+{
+	note_connections.drop_connections ();
+	_last_seen_external_midi_note = -1;
+
+	for (auto const & p : *_input->ports()) {
+		std::shared_ptr<MidiPort> mp = std::dynamic_pointer_cast<MidiPort> (p);
+		if (mp) {
+			mp->NoteOn.connect_same_thread (note_connections, std::bind (&MidiTrack::note_on_handler, this, _1));
+		}
+	}
+
+	Track::input_change_handler (change, src);
+}
+
+void
+MidiTrack::note_on_handler (int notenum)
+{
+	_last_seen_external_midi_note = notenum;
+}
