@@ -782,27 +782,32 @@ icon_transport_loop_mode (cairo_t* cr, const int width, const int height, const
 static void
 icon_transport_auto_return (cairo_t* cr, const int width, const int height, const uint32_t fg_color)
 {
-	static const double degrees = M_PI / 180.0;
-
-	double linew = std::max(1, std::min(width, height) / 8);
-
 	const double x = width * .5;
 	const double y = height * .5;
-	const double r = std::min (x, y)/3.;
+	const double r = std::min (x, y);
 
-	cairo_move_to (cr, x, y-r); //top left
-	cairo_line_to (cr, x, y-r);  //top line
-	cairo_arc (cr, x+r, y, r, -90 * degrees, 90 * degrees);  //right arc
-	cairo_line_to (cr, x-r, y+r);  //bottom line
-	VECTORICONSTROKE(linew, fg_color);
+	cairo_arc (cr, x, y, r * .58, -0.75 * M_PI, 0.50 * M_PI);
+	cairo_rel_line_to(cr, -0.45 * r , 0);
+	cairo_rel_line_to(cr, 0, -0.28 * r);
+	cairo_rel_line_to(cr, 0.45 * r, 0);
+	cairo_arc_negative (cr, x, y, r * .30, 0.50 * M_PI, -0.75 * M_PI);
 
-	linew/=2.0;
-	float r2 = r/1.5;
-	cairo_move_to (cr, x, y-r);
-	cairo_line_to (cr, x+r2, y-r-r2);  //arrow
-	cairo_move_to (cr, x, y-r);
-	cairo_line_to (cr, x+r2, y-r+r2);  //arrow
-	VECTORICONSTROKE(linew, fg_color);
+	VECTORICONSTROKEFILLFG (1.0);
+
+#define ARCARROW(rad, ang) \
+	x + (rad)*sin ((ang)*2.0 * M_PI), y + (rad)*cos ((ang)*2.0 * M_PI)
+
+	cairo_move_to (cr, ARCARROW (r * .30, .5));
+	cairo_line_to (cr, ARCARROW (r * .11, .5));
+	cairo_line_to (cr, ARCARROW (r * .55, 0.62));
+	cairo_line_to (cr, ARCARROW (r * .74, .5));
+	cairo_line_to (cr, ARCARROW (r * .58, .5));
+
+	cairo_set_source_rgba (cr, 0, 0, 0, 1.0);
+	cairo_stroke_preserve (cr);
+	cairo_close_path (cr);
+	VECTORICONSTROKEFILLFG (1.0);
+#undef ARCARROW
 }
 
 /** triangle phead between brackets  */
