@@ -16,10 +16,11 @@ test -f gtk2_ardour/wscript || exit 1
 : ${MAKEFLAGS=-j4}
 : ${TMPDIR=/var/tmp}
 : ${SRCCACHE=/var/tmp/winsrc}  # source-code tgz cache
+: ${CURLOPT="-s -S --retry-connrefused --retry 3"}
 
 : ${HARRISONCHANNELSTRIP=harrison_channelstrip}
 : ${HARRISONLV2=harrison_lv2s-n}
-: ${HARRISONDSPURL=https://rsrc.harrisonconsoles.com/plugins/releases/public}
+: ${HARRISONDSPURL=https://builder.harrisonconsoles.com/pub/dsp}
 
 # see also wscript, video_tool_paths.cc, bundle_env_mingw.cc
 # registry keys based on this are used there
@@ -247,10 +248,19 @@ cp gtk2_ardour/icons/ArdourBug.ico $DESTDIR/share/
 # replace default cursor with square version (sans hotspot file)
 cp gtk2_ardour/icons/cursor_square/* $DESTDIR/share/${LOWERCASE_DIRNAME}/icons/
 
-# clean build-dir after depoyment
+# clean build-dir after deployment
 echo " === bundle completed, cleaning up"
 ./waf uninstall
 find $DESTDIR -name "*.dll.a" -print0 | xargs -0 -r rm
+
+# no need for MIDNAM
+if test -n "$VBM" -o -n "$LIVETRAX"; then
+	rm -rf $DESTDIR/share/*/patchfiles
+fi
+if test -n "$LIVETRAX"; then
+	rm -rf $DESTDIR/share/*/scripts
+fi
+
 echo " === complete"
 du -sh $DESTDIR
 
@@ -259,7 +269,7 @@ du -sh $DESTDIR
 
 if test -n "$WITH_HARVID"; then
 	echo " === Including harvid"
-	HARVID_VERSION=$(curl -s -S http://ardour.org/files/video-tools/harvid_version.txt)
+	HARVID_VERSION=$(curl ${CURLOPT} http://ardour.org/files/video-tools/harvid_version.txt)
 
 	rsync -a -q --partial \
 		rsync://ardour.org/video-tools/harvid_${WARCH}-${HARVID_VERSION}.tar.xz \
@@ -271,7 +281,7 @@ fi
 
 if test -n "$WITH_XJADEO"; then
 	echo " === Including video-monitor"
-	XJADEO_VERSION=$(curl -s -S http://ardour.org/files/video-tools/xjadeo_version.txt)
+	XJADEO_VERSION=$(curl ${CURLOPT} http://ardour.org/files/video-tools/xjadeo_version.txt)
 
 	rsync -a -q --partial \
 		rsync://ardour.org/video-tools/xjadeo_${WARCH}-${XJADEO_VERSION}.tar.xz \
@@ -327,7 +337,7 @@ if test x$WITH_GMSYNTH != x ; then
 	echo "Adding General MIDI Synth LV2"
 
 	for proj in x42-gmsynth; do
-		X42_VERSION=$(curl -s -S http://x42-plugins.com/x42/win/${proj}.latest.txt)
+		X42_VERSION=$(curl ${CURLOPT} http://x42-plugins.com/x42/win/${proj}.latest.txt)
 		rsync -a -q --partial \
 			rsync://x42-plugins.com/x42/win/${proj}-lv2-${WARCH}-${X42_VERSION}.zip \
 			"${SRCCACHE}/${proj}-lv2-${WARCH}-${X42_VERSION}.zip"
@@ -341,7 +351,7 @@ if test x$WITH_COMMERCIAL_X42_LV2 != x ; then
 	echo "Adding commercial x42 Plugins"
 
 	for proj in x42-meters x42-eq x42-whirl; do
-		X42_VERSION=$(curl -s -S http://x42-plugins.com/x42/win/${proj}.latest.txt)
+		X42_VERSION=$(curl ${CURLOPT} http://x42-plugins.com/x42/win/${proj}.latest.txt)
 		rsync -a -q --partial \
 			rsync://x42-plugins.com/x42/win/${proj}-lv2-${WARCH}-${X42_VERSION}.zip \
 			"${SRCCACHE}/${proj}-lv2-${WARCH}-${X42_VERSION}.zip"
@@ -366,7 +376,7 @@ if test x$WITH_GRATIS_X42_LV2 != x ; then
 			fi
 		fi
 
-		X42_VERSION=$(curl -s -S http://x42-plugins.com/x42/win/${proj}.latest.txt)
+		X42_VERSION=$(curl ${CURLOPT} http://x42-plugins.com/x42/win/${proj}.latest.txt)
 		rsync -a -q --partial \
 			rsync://x42-plugins.com/x42/win/${proj}-lv2-${WARCH}-${X42_VERSION}.zip \
 			"${SRCCACHE}/${proj}-lv2-${WARCH}-${X42_VERSION}.zip"
@@ -379,7 +389,7 @@ if test x$WITH_HARRISON_LV2 != x ; then
 
 	echo "Including Harrison LV2s"
 
-	curl -s -S --fail -# \
+	curl ${CURLOPT} --fail -# \
 		-z "${SRCCACHE}/${HARRISONLV2}.${WARCH}.zip" \
 		-o "${SRCCACHE}/${HARRISONLV2}.${WARCH}.zip" \
 		"${HARRISONDSPURL}/${HARRISONLV2}.${WARCH}.zip"
@@ -391,7 +401,7 @@ if test x$WITH_HARRISON_VBM != x ; then
 
 	echo "Including Harrison VBM Channelstrip LV2"
 
-	curl -s -S --fail -# \
+	curl ${CURLOPT} --fail -# \
 		-z "${SRCCACHE}/harrison_vbm.${WARCH}.zip" \
 		-o "${SRCCACHE}/harrison_vbm.${WARCH}.zip" \
 		"${HARRISONDSPURL}/harrison_vbm.${WARCH}.zip"
@@ -405,7 +415,7 @@ if test -n "$MIXBUS"; then
 	echo "Deploying Harrison Mixbus Channelstrip"
 
 	mkdir -p $ALIBDIR/ladspa/strip
-	curl -s -S --fail -# \
+	curl ${CURLOPT} --fail -# \
 		-z "${SRCCACHE}/${HARRISONCHANNELSTRIP}.${WARCH}.dll" \
 		-o "${SRCCACHE}/${HARRISONCHANNELSTRIP}.${WARCH}.dll" \
 		"${HARRISONDSPURL}/${HARRISONCHANNELSTRIP}.${WARCH}.dll"
@@ -415,7 +425,7 @@ if test -n "$MIXBUS"; then
 
 	echo "Deploying Harrison Vamp Plugins"
 	mkdir -p $ALIBDIR/vamp
-	curl -s -S --fail -# \
+	curl ${CURLOPT} --fail -# \
 		-z "${SRCCACHE}/harrison_vamp.${WARCH}.dll" \
 		-o "${SRCCACHE}/harrison_vamp.${WARCH}.dll" \
 		"${HARRISONDSPURL}/harrison_vamp.${WARCH}.dll"
@@ -424,10 +434,10 @@ if test -n "$MIXBUS"; then
 		"$ALIBDIR/vamp/harrison_vamp.dll"
 
 	# Mixbus Bundled Media Content
-	curl -s -S --fail -#  \
+	curl ${CURLOPT} --fail -#  \
 		-z "${SRCCACHE}/MixbusBundledMedia.zip" \
 		-o "${SRCCACHE}/MixbusBundledMedia.zip" \
-		"http://builder.harrisonconsoles.com/pub/share/MixbusBundledMedia.zip"
+		"https://builder.harrisonconsoles.com/pub/share/MixbusBundledMedia.zip"
 
 	if test -f "${SRCCACHE}/MixbusBundledMedia.zip"; then
 		echo "Adding Mixbus Bundled Content"
@@ -436,7 +446,7 @@ if test -n "$MIXBUS"; then
 	fi
 elif test -z "$LIVETRAX" -a -z "$VBM"; then
 	echo "Fetching Ardour bundled media"
-	curl -s -S --fail -#  \
+	curl ${CURLOPT} --fail -#  \
 		-z "${SRCCACHE}/ArdourBundledMedia.zip" \
 		-o "${SRCCACHE}/ArdourBundledMedia.zip" \
 		"http://stuff.ardour.org/loops/ArdourBundledMedia.zip"
@@ -452,9 +462,9 @@ fi
 
 if test x$DEMO_SESSION_URL != x ; then
 	mkdir -p $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions
-	DEMO_SESSIONS=$(curl -s -S --fail $DEMO_SESSION_URL/index.txt)
+	DEMO_SESSIONS=$(curl ${CURLOPT} --fail $DEMO_SESSION_URL/index.txt)
 	for demo in $DEMO_SESSIONS; do
-		curl -s -S --fail -# -o $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions/$demo $DEMO_SESSION_URL/$demo
+		curl ${CURLOPT} --fail -# -o $DESTDIR/share/${LOWERCASE_DIRNAME}/sessions/$demo $DEMO_SESSION_URL/$demo
 	done
 fi
 
@@ -558,7 +568,7 @@ Section "${PROGRAM_NAME}${PROGRAM_VERSION} (required)" SecMainProg
   WriteRegDWORD HKLM "Software\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\${PRODUCT_ID}-${WARCH}" "NoRepair" 1
   WriteUninstaller "\$INSTDIR\uninstall.exe"
   CreateShortCut "\$INSTDIR\\${PROGRAM_NAME}${PROGRAM_VERSION}.lnk" "\$INSTDIR\\bin\\${PRODUCT_EXE}" "" "\$INSTDIR\\bin\\${PRODUCT_EXE}" 0
-  \${registerExtension} "\$INSTDIR\\bin\\${STATEFILE_SUFFIX}" ".${PRODUCT_NAME}" "${PROGRAM_NAME} Session"
+  \${registerExtension} "\$INSTDIR\\bin\\${PRODUCT_EXE}" ".${STATEFILE_SUFFIX}" "${PROGRAM_NAME} Session"
 SectionEnd
 EOF
 
